BaT_Theme.getCookie = function (cookieName) {
    var index = 0,
        parts = document.cookie.split(';'),
        value = '';

    for (index = 0; index < parts.length; index++) {
        value = jQuery.trim(parts[index]);

        if (value.indexOf(cookieName) == 0 && '=' === value.substr(cookieName.length, 1)) {
            // addition of 1 to account for equals sign
            return value.substr(cookieName.length + 1, value.length);
        }
    }

    return null;
};

BaT_Theme.setCookie = function (cookieName, cookieValue, expiresHours) {
    var date = new Date(), expires;

    date.setTime(date.getTime() + Math.ceil(expiresHours * 60 * 60 * 1000));
    expires = "expires=" + date.toUTCString();

    document.cookie = cookieName + "=" + cookieValue + "; " + expires + "; path=/";
};

BaT_Theme.getTickableFunction = function ($element, callback) {
    var $element = jQuery($element),
        $placeh = jQuery('<span></span>').hide();

    $placeh.insertAfter($element);

    return function (parts) {
        var secondsRemaining = jQuery.countdown.periodsToSeconds(parts);

        if (callback && callback.call) { callback.call(this, parts); }

        if ($element.data('processed_final')) { return; }

        // Less than 2 minutes
        if (secondsRemaining <= 120) {
            $element.addClass('blinker').data('processed_final', true);
            $placeh.hide();
        }

        if ($element.data('processed_second')) { return; }

        // Less than 1 hour
        if (secondsRemaining < 3600) {
            $element.data('processed_second', true).data('processed', true)
            $placeh.hide();

            setTimeout(function () {
                $element.countdown('option', 'compact', true).countdown('option', 'layout', '{mn}:{snn}').addClass('text-primary');
            }, 10);
        }

        if ($element.data('processed')) { return; }

        // Less than 24 hours
        if (secondsRemaining < 24 * 60 * 60) {
            $element.data('processed', true);
            $placeh.hide();

            setTimeout(function () {
                $placeh.hide();
                $element.show().countdown('option', { compact: true, layout: '{h<}{hn}:{h>}{mnn}:{snn}' }).addClass('text-primary');
            }, 10);

        } else {
            $element.hide();
            $placeh.show();

            // More than 24 hours
            var dateFinal = new Date($element.countdown('option', 'until').getTime()),
                dateCurrent = new Date(),
                days = 0;

            while (dateCurrent < dateFinal) {
                days++;

                dateCurrent.setDate(dateCurrent.getDate() + 1);
                dateCurrent.setHours(dateFinal.getHours());
                dateCurrent.setMinutes(dateFinal.getMinutes());
                dateCurrent.setSeconds(dateFinal.getSeconds());
                dateCurrent.setMilliseconds(dateFinal.getMilliseconds());
            }

            if (1 === days) {
                $placeh.text(days + ' day');
            } else {
                $placeh.text(days + ' days');
            }
        }
    };
};

BaT_Theme.addBid = function (bidData) {
    var $list = jQuery('.recent-bids-list'),
        $links = jQuery('.recent-bids-image-links'),
        $item = jQuery(bidData.markup_bid),
        $link = jQuery(bidData.markup_bid_image),
        $curr = $list.find('[data-listing="' + $item.data('listing') + '"]'),
        $currl = $links.find('[data-listing="' + $item.data('listing') + '"]'),
        image = new Image();

    var afterPreload = function () {
        var callback = function () {
            $item.find('.recent-bids-list-item-datum-inner:not(.recent-bids-list-item-datum-inner-placeholder)').slideDown(400);

            setTimeout(function () {
                $links.animate({ opacity: 0 }, 200, function () {
                    $currl.remove();
                    $links.prepend($link);
                    $links.animate({ opacity: 1 }, 200);

                    BaT_Theme.recentBidsCountdowns();
                });
            }, 450);
        };

        $item.find('.recent-bids-list-item-datum-inner:not(.recent-bids-list-item-datum-inner-placeholder)').css('display', 'block').hide().end().prependTo($list);

        jQuery('.recent-bids-image-link:nth-child(2)').css('margin-right', 0);
        jQuery('.recent-bids-image-link:nth-child(3)').hide();

        if ($curr.size() > 0) {
            $curr.find('.recent-bids-list-item-datum-inner:visible').fadeOut(400);

            setTimeout(function () {
                $curr.find('.recent-bids-list-item-datum-remaining .recent-bids-list-item-datum-inner').countdown('destroy').end().remove();

                callback();
            }, 450);
        } else {
            // Account for when there isn't a previous bid
            callback();
        }
    };

    image.onload = afterPreload;
    image.src = $link.find('img').attr('src');
};

BaT_Theme.recentBidsCountdowns = function () {
    jQuery('.recent-bids-list-item .recent-bids-list-item-datum-remaining .recent-bids-list-item-datum-inner[data-recent_bid_auction_ends]').filter(function (index, element) {
        return true !== jQuery(element).data('recent_bid_countdown_initialized');
    }).each(function (index, element) {
        var $element = jQuery(element),
            $placeh = $element.siblings('.recent-bids-list-item-datum-inner-placeholder'),
            until = new Date($element.data('recent_bid_auction_ends') * 1000),
            options = {};

        options = {
            alwaysExpire: true,
            expiryText: 'Ended',
            format: 'ymwdhMS',
            labels: ['years', 'months', 'weeks', 'days', 'hours', 'minutes', 'seconds'],
            labels1: ['year', 'month', 'week', 'day', 'hour', 'minute', 'second'],
            layout: ' ',
            until: until,

            onExpiry: function () {
                var $done = $element.parents('.recent-bids').find('[data-listing="' + $element.parents('.recent-bids-list-item').data('listing') + '"]');

                setTimeout(function () {
                    $done.fadeOut(400, function () {
                        $element.countdown('destroy');
                        $done.remove();
                    });
                }, 1);
            },

            onTick: function (parts) {
                var secondsRemaining = jQuery.countdown.periodsToSeconds(parts);


                if ($element.data('processed_final')) { return; }

                // Less than 2 minutes
                if (secondsRemaining <= 120) {
                    $element.addClass('blinker').data('processed_final', true);
                }

                if ($element.data('processed_second')) { return; }

                // Less than 1 hour
                if (secondsRemaining <= 3600) {
                    $element.data('processed_second', true).data('processed', true)

                    setTimeout(function () {
                        $element.countdown('option', 'compact', true).countdown('option', 'layout', '{mn}:{snn}').addClass('recent-bids-list-item-datum-inner-final');
                    }, 100);
                }

                if ($element.data('processed')) { return; }

                // Less than 24 hours
                if (secondsRemaining < 24 * 60 * 60) {
                    $element.data('processed', true);

                    setTimeout(function () {
                        $element.show().countdown('option', { compact: true, layout: '{h<}{hn}:{h>}{mnn}:{snn}' }).addClass('recent-bids-list-item-datum-inner-final');
                        $placeh.hide();
                    }, 100);

                } else {
                    $element.hide();
                    $placeh.show();

                    // More than 24 hours
                    var dateFinal = new Date($element.countdown('option', 'until').getTime()),
                        dateCurrent = new Date(),
                        days = 0;

                    while (dateCurrent < dateFinal) {
                        days++;

                        dateCurrent.setDate(dateCurrent.getDate() + 1);
                        dateCurrent.setHours(dateFinal.getHours());
                        dateCurrent.setMinutes(dateFinal.getMinutes());
                        dateCurrent.setSeconds(dateFinal.getSeconds());
                        dateCurrent.setMilliseconds(dateFinal.getMilliseconds());
                    }

                    if (1 === days) {
                        $placeh.text(days + ' day');
                    } else {
                        $placeh.text(days + ' days');
                    }
                }
            }
        };

        $element.countdown(options);

        $element.data('recent_bid_countdown_initialized', true);
    });
};

BaT_Theme.featuredListingsCountdown = function () {
    jQuery('.featured-listing-meta-value[data-featured_listing][data-featured_listing_ends]').each(function (index, element) {
        var $element = jQuery(element),
            $placeh = $element.siblings('[data-featured_placeholder]'),
            until = new Date($element.data('featured_listing_ends') * 1000),
            options = {};

        options = {
            alwaysExpire: true,
            expiryText: 'Ended',
            format: 'ymwdhMS',
            labels: ['years', 'months', 'weeks', 'days', 'hours', 'minutes', 'seconds'],
            labels1: ['year', 'month', 'week', 'day', 'hour', 'minute', 'second'],
            layout: '{dn} {dl}',
            until: until,

            onExpiry: function () {

            },

            onTick: function (parts) {
                var secondsRemaining = jQuery.countdown.periodsToSeconds(parts);

                if ($element.data('processed_final')) { return; }

                // Less than 2 minutes
                if (secondsRemaining <= 120) {
                    $element.addClass('blinker').data('processed_final', true);
                }

                if ($element.data('processed_second')) { return; }

                // Less than 1 hour
                if (secondsRemaining <= 3600) {
                    $element.data('processed_second', true).data('processed', true)

                    setTimeout(function () {
                        $element.countdown('option', 'compact', true).countdown('option', 'layout', '{mn}:{snn}').addClass('recent-bids-list-item-datum-inner-final');
                    }, 100);
                }

                if ($element.data('processed')) { return; }

                // Less than 24 hours
                if (secondsRemaining < 24 * 60 * 60) {
                    $element.data('processed', true);

                    setTimeout(function () {
                        $element.show().countdown('option', { compact: true, layout: '{h<}{hn}:{h>}{mnn}:{snn}' }).addClass('recent-bids-list-item-datum-inner-final');
                        $placeh.hide();
                    }, 100);

                } else {
                    $element.hide();
                    $placeh.show();

                    // More than 24 hours
                    var dateFinal = new Date($element.countdown('option', 'until').getTime()),
                        dateCurrent = new Date(),
                        days = 0,
                        dayInMilliseconds = 24 * 60 * 60 * 1000;

                    dateFinal.setHours(0);
                    dateFinal.setMinutes(0);
                    dateFinal.setSeconds(0);
                    dateFinal.setMilliseconds(0);

                    dateCurrent.setHours(0);
                    dateCurrent.setMinutes(0);
                    dateCurrent.setSeconds(0);
                    dateCurrent.setMilliseconds(0);

                    while (dateCurrent.getTime() < dateFinal.getTime()) {
                        days++;
                        dateCurrent.setTime(dateCurrent.getTime() + dayInMilliseconds);
                    }

                    if (1 === days) {
                        $placeh.text(days + ' day');
                    } else {
                        $placeh.text(days + ' days');
                    }
                }
            }
        };

        $element.countdown(options);
    });
};

if ('no' === BaT_Theme.loggedIn) {
    /**
     * The following basically detects if the user is returning after more than
     * an hour away from the site or if they are fresh to the site. If either of
     * those cases are two, then increment the visit counter. If the visit counter
     * is equal to 2 then run some functionality.
     */
    (function () {
        var trackerAlt = BaT_Theme.getCookie('bat_tracking_data_alt'),
            trackerMain = BaT_Theme.getCookie('bat_tracking_data');

        if ((null === trackerAlt && null === trackerMain) && 'object' === typeof JSON && 'function' === typeof JSON.stringify) {
            BaT_Theme.setCookie('bat_tracking_data_alt', JSON.stringify({
                conversion: 0,
                datetime: Math.ceil((new Date()).getTime() / 1000),
                redirect: document.location.href,
                referrer: document.referrer,
            }), 24 * 365 * 3);
        }
    }());
}

jQuery(document).ready(function ($) {
    var iPad = navigator.userAgent.match(/iPad/i) != null;

    if (iPad) {
        $('body').addClass('ua-ipad');
    }

    // Handle iPad click
    $(document).on('click', '.header-menu > li.menu-item > a', function (event) {
        if (iPad && $(this).siblings('.sub-menu').length) {
            $('.header-menu > li.menu-item').not($(this).parent()).removeClass('menu-item--open');
            event.preventDefault();
            $(this).parent().toggleClass('menu-item--open');
        }
    });

    $('.bat-listings-users-form-notifications .toggle-switch').click(function(event) {
        event.preventDefault();

        var $input = $(this).children('input'),
            active = $input.val(),
            offValue = ($input.data('offValue') || 'off'),
            onValue = ($input.data('onValue') || 'on');

        if (onValue == active) {
            $(this).children('input').val(offValue);
            $(this).removeClass('on').addClass('off');
        } else {
            $(this).children('input').val(onValue);
            $(this).removeClass('off').addClass('on');
        }
    });

    var active = $('#control-group-text-message-notifications input').val();

    $('#control-group-text-message-notifications-number').toggle('on' === active);

    $('#control-group-text-message-notifications .toggle-switch').click(function(event) {
        active = $(this).children('input').val();

        $('#control-group-text-message-notifications-number').toggle('on' === active);
    });

    $('.bat-listings-users-form-notifications').submit(function(event) {
        var toggle = $('#bat-listings-users-text-message-notifications').val(),
            number = $('#bat-listings-users-text-message-notifications-number').val().replace(/[^\d]/g, '');

        $('#control-group-text-message-notifications-number').find('.control-group-error').remove();

        if('on' === toggle && 10 !== number.length) {
            event.preventDefault();

            $('#control-group-text-message-notifications-number')
                .find('.control-group-instructions')
                .after($('<label for="bat-listings-listings-text-message-notifications-number" class="control-group-error"></label>').text('You must provide a 10-digit number.'));
        }
    });

    // Listing viewed
    (function () {
        if (BaT_Theme.trackViewUrl) {
            $.post(BaT_Theme.trackViewUrl);
        } else if (BaT_Theme.post && 'bat-listing' === BaT_Theme.type) {
            $.post(
                BaT_Theme.ajaxUrl,
                {
                    action: BaT_Theme.ajaxActionListingView,
                    post: BaT_Theme.post
                }
            );
        }
    }());

    // Listings overview page - list and grid view
    (function () {
        $('.toolbar-button-grid-view, .toolbar-button-list-view').click(function (event) {
            event.preventDefault();

            var $button = $(this),
                $auctions = $('.auctions-current:not(.auctions-current-sorter)'),
                loggedIn = $button.parents('body.loggedin').size() > 0;

            $button.siblings().removeClass('toolbar-button-active').end().addClass('toolbar-button-active');
            $auctions.toggleClass('auctions-list', $button.is('.toolbar-button-list-view'));

            if ($button.is('.toolbar-button-grid-view')) {
                $('.auctions-item-sorter-link[data-orderby="until"]').data('ordered', -1).trigger('click');
            }

            if (loggedIn) {
                $.post(
                    BaT_Theme.ajaxUrl,
                    {
                        action: BaT_Theme.ajaxActionUserViewToggle,
                        view: $button.is('.toolbar-button-list-view') ? 'list' : 'grid'
                    },
                    function (data, status) { },
                    'json'
                );
            }
        });
    }());

    // Listings overview page - countdowns
    function countdowns() {
        $('.auctions-current .auctions-item-container, .premium-item').each(function (index, auction) {
            var $auction = $(auction),
                $countdown = $auction.find('[data-countdown]'),
                $progress = $auction.find('.auctions-item-progress-bar'),
                layout = '{d<}{dn} {dl}, {d>} {h<}{hn}:{h>}{m<}{mnn}:{m>}{snn}',
                expiration = function () { },
                progression = function (parts) { },
                until = new Date($countdown.data('until') * 1000);

            expiration = function () {
                $auction.fadeOut(500, function () {
                    $auction.remove();
                    // auctionsDynamicFilter.reloadAuctions(false);
                });
            };

            progression = function (parts) {
                var seconds = jQuery.countdown.periodsToSeconds(parts),
                    extension = parseFloat(BaT_Theme.extensionWindowSeconds),
                    width = ((Math.min(extension, seconds) / extension) * 100) + '%';

                $progress.toggleClass('listing-available-progress-bar-counting', seconds <= extension)
                    .toggleClass('listing-available-progress-bar-finalmin', seconds <= 60)
                    .css('width', width);
            };

            $countdown.countdown({
                layout: layout,
                onExpiry: expiration,
                onTick: BaT_Theme.getTickableFunction($countdown, progression),
                until: until
            });
        });
    }

    // Listings overview page - initialize countdowns
    (function () {
        countdowns();
    }());

    // Listings overview page - listing watch
    (function () {
        $(document).on('click', '.loggedin .auctions-item-watch, .loggedin .auctions-item-watchers', function (event) {
            event.preventDefault();

            var $link = $(this).toggleClass('auctions-item-watched'),
                $item = $(this).parents('.auctions-item-container');

            $item.data('watched', 0 === $item.data('watched') ? 1 : 0);

            if($('.toolbar').length) {
                auctionsInlineFilter.filter();
            }

            $.post(
                $link.attr('href'),
                { ajax: true },
                function (data, status) {
                    // What should happen here?
                },
                'json'
            );
        });
    }());

    // Listings overview page - dynamic filter
    var auctionsDynamicFilter = (function () {
        var filter = null,
            request = null,
            search = null;

        filter = function (grey) {
            var $cat = $('.toolbar-cats-field'),
                $view = $('.toolbar-views').find('.toolbar-button-active'),
                $watched = $('.toolbar-button-watched');

            // Make sure we're normalizing to a bool
            grey = !!grey;

            if (grey) {
                $('.auctions-data').addClass('auctions-data-fetching');
            }

            if (null !== request) {
                request.abort();
            }

            request = $.get(
                BaT_Theme.urlListings,
                {
                    batcat: 0,
                    batview: $view.is('.toolbar-button-list-view') ? 'list' : 'grid',
                    batwatched: false
                },
                function (data, status) {
                    // Construct new DOM to fetch from
                    var $page = $(data);

                    // Request has finished
                    request = null;

                    // Destroy the countdown items
                    $('.auctions-data').find('[data-countdown]').countdown('destroy');

                    // Replace content
                    $('.auctions-data').replaceWith($page.find('.auctions-data'));

                    // Fire up the new countdowns
                    countdowns();
                },
                'html'
            );
        };

        var exports = {};
        exports.reloadAuctions = filter;
        return exports;
    }());

    // Listings overview page - sort by location options
    var auctionsDynamicLocationChoices = (function () {
        var updateLocationOptions = function () {
            var $sortSelect = $('.toolbar-sort-select'),
                sortLastChoice = $sortSelect.val(),
                userLocation = BaT_Theme.getUserLocation();

            // Clear existing location options
            $sortSelect.find("option[value^='location']").remove();

            if (userLocation) {
                var zipCode = userLocation['zipCode'];

                $sortSelect.append($('<option>', {
                    value: 'location',
                    text: BaT_Theme.closestToZipSavedText + zipCode,
                }));

                $sortSelect.append($('<option>', {
                    value: 'location-set',
                    text: BaT_Theme.closestToZipChangeText,
                }).prepend('&nbsp;&nbsp;&nbsp;&nbsp;'));
            } else {
                $sortSelect.append($('<option>', {
                    value: 'location-set',
                    text: BaT_Theme.closestToZipNoneSavedText,
                }));
            }
        }

        return {
            updateLocationOptions: updateLocationOptions
        };
    }());

    // Listings overview page - sorting
    var auctionsDynamicSort = (function () {
        var $sortSelect = $('.toolbar-sort-select');

        auctionsDynamicLocationChoices.updateLocationOptions();

        // Store the previous value, in case it needs to be reverted if location prompt is canceled
        $sortSelect.data('previous-value', $sortSelect.val());

        $sortSelect.change(function (event) {
            event.preventDefault();

            var $select = $(this),
                $current = $('.auctions-current:not(.auctions-current-sorter)'),
                $auctions = $current.find('.auctions-item-container:not(.auctions-item-container-sorter)'),
                value = $select.val(),
                parts = value.split('|'),
                orderby = ['timestamp', 'price', 'location', 'location-set', 'recommended', 'plus', 'whiteglove'].indexOf(parts[0]) === -1 ? '' : parts[0],
                ordered = parts.length > 1 ? parseInt(parts[1]) : -1,
                selectPreviousValue = $sortSelect.data('previous-value');

            $sortSelect.data('previous-value', $sortSelect.val());
            $('.toolbar-sort-select').find('option[value="' + value + '"]').prop('selected', true);
            $('.auctions-item-sorter-select-text').text($select.find('option:selected').text());

            if (orderby === 'location-set') {
                // Prompt user for location and call back into this function when it's complete
                return BaT_Theme.promptUserLocation(function () {
                    var location = BaT_Theme.getUserLocation();

                    auctionsDynamicLocationChoices.updateLocationOptions();

                    if (location === false) {
                        $select.val(selectPreviousValue);
                    } else {
                        $select.val('location');
                    }

                    $select.trigger('change');
                }, function () {
                    $select.val(selectPreviousValue);

                    $select.trigger('change');
                });
            } else if (orderby === 'location') {
                // Pre-compute the distances to each listing
                var location = BaT_Theme.getUserLocation(),
                    userLat = location ? location.lat : false,
                    userLon = location ? location.lon : false;

                $auctions.each(function (index, auction) {
                    var $auction = $(auction),
                        auctionLat = $auction.data('lat'),
                        auctionLon = $auction.data('lon');

                    if (userLat && userLon && auctionLat && auctionLon) {
                        var distanceMiles = auctionsDynamicSort.distanceBetween(userLat, userLon, auctionLat, auctionLon, /* unit */ 'miles');
                        $auction.data('distance-miles', distanceMiles);
                    }
                });

                // Replace 'Ends In' data with 'Distance' on auctions
                $auctions.each(function (index, auction) {
                    var $auction = $(auction),
                        $distanceSection = $auction.find('.auctions-item-meta-block-distance'),
                        $endsInSection = $auction.find('.auctions-item-meta-block-ends-in'),
                        distance = $auction.data('distance-miles'),
                        auctionCountry = $auction.data('country');

                    if (distance === undefined && auctionCountry.length === 0) {
                        $distanceSection.find('.auctions-item-meta-value-distance').text(BaT_Theme.closestToZipNADistanceText);
                    } else if (distance === undefined) {
                        $distanceSection.find('.auctions-item-meta-value-distance').text(auctionCountry);
                    } else {
                        var distanceFormatted = auctionsDynamicSort.formatDistanceInMiles(distance);
                        $distanceSection.find('.auctions-item-meta-value-distance').text(distanceFormatted);
                    }

                    $endsInSection.hide();
                    $distanceSection.show();
                });
            } else {
                // For non-location sorting, hide distance and show ending time
                $auctions.find('.auctions-item-meta-block-distance').hide();
                $auctions.find('.auctions-item-meta-block-ends-in').show();
            }

            $auctions.sort(function (a, b) {
                var idA, idB, valueA, valueB;

                if (orderby === 'timestamp' || orderby === 'plus' || orderby === 'whiteglove') {
                    valueA = $(a).data('timestamp');
                    valueB = $(b).data('timestamp');
                } else if (orderby === 'price') {
                    valueA = $(a).data('price');
                    valueB = $(b).data('price');
                } else if (orderby === 'location') {
                    valueA = $(a).data('distance-miles');
                    valueB = $(b).data('distance-miles');

                    // Order auctions without a distance (e.g. non-US auctions) last
                    if (valueA === undefined && valueB !== undefined) { return 1; }
                    if (valueB === undefined && valueA !== undefined) { return -1; }

                    // If both auctions do not have a distance (e.g. non-US auctions), order alphabetically by country
                    if (valueA === undefined && valueB === undefined) {
                        valueA = $(a).data('country');
                        valueB = $(b).data('country');

                        // Order auctions without a country (e.g. generated auctions) last
                        if (valueA.length === 0 && valueB.length > 0) { return 1; }
                        if (valueB.length === 0 && valueA.length > 0) { return -1; }
                    }
                } else if (orderby === 'recommended') {
                    idA = $(a).data('update');
                    idB = $(b).data('update');

                    valueA = jQuery.inArray(idA, BaT_Theme.recommendedIds);
                    valueB = jQuery.inArray(idB, BaT_Theme.recommendedIds);

                    if (-1 === valueA) {
                        valueA = $auctions.size() + 1;
                    }

                    if (-1 === valueB) {
                        valueB = $auctions.size() + 1;
                    }
                }

                return valueA === valueB ? 0 : (valueA < valueB ? ordered : -1 * ordered);
            });

            $auctions.each(function (index, auction) {
                $current.append($(auction).show());
            });

            if(orderby === 'plus' || orderby === 'whiteglove') {
                $auctions.filter(':not([data-processing_type="' + orderby + '"])').hide();
            }
        });

        distanceBetween = function (lat1, lon1, lat2, lon2) {
            unit = (typeof unit === 'undefined') ? 'miles' : unit;

            var deg2rad = function (deg) {
                return deg * (Math.PI / 180)
            };

            var R = 6371; // Radius of the earth in km
            var dLat = deg2rad(lat2 - lat1);
            var dLon = deg2rad(lon2 - lon1);
            var a = Math.sin(dLat / 2) * Math.sin(dLat / 2) +
                Math.cos(deg2rad(lat1)) * Math.cos(deg2rad(lat2)) *
                Math.sin(dLon / 2) * Math.sin(dLon / 2);
            var c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a));
            var distanceInKm = R * c;

            if (unit === 'miles') {
                return distanceInKm * 0.621371;
            } else {
                return distanceInKm;
            }
        };

        formatDistanceInMiles = function (distance) {
            var approximateDrivingDistance = distance * 1.2;

            if (approximateDrivingDistance < 10) {
                return '< 10 ' + BaT_Theme.closestToZipMilesText;
            } else if (approximateDrivingDistance < 1000) {
                // Round values up to the nearest 10, e.g. 76 becomes 80
                var distanceRounded = Math.ceil(approximateDrivingDistance / 10) * 10;
                return distanceRounded + ' ' + BaT_Theme.closestToZipMilesText;
            } else {
                // Round values up to the nearest 100, e.g. 1015 becomes 1100
                var distanceRounded = Math.ceil(approximateDrivingDistance / 100) * 100;
                return distanceRounded + ' ' + BaT_Theme.closestToZipMilesText;
            }
        }

        return {
            sort: function () { $('.toolbar-sort-select').first().trigger('change'); },
            distanceBetween: distanceBetween,
            formatDistanceInMiles: formatDistanceInMiles
        };
    }());

    // Listings overview page - filtering
    var auctionsInlineFilter = (function () {
        var filtered = [], handle = null, registered = false, filter = function () {
            var $auctions, $current, $filtered, terms, value, watched, resultsURL, params = {};

            // Clear the handle
            handle = null;

            // Fetch the terms that must be found
            value = $.trim($('.post-type-archive-bat-listing .toolbar-search-input').first().val());
            terms = value.split(/\s+/).filter(function (item) { return '' !== item; });

            if ('' !== value) { filtered.push(value); }

            // Fetch whether "watched only" is toggled on
            watched = !!$('.post-type-archive-bat-listing .toolbar-button-watched').data('watched');

            // Update the 'Want completed results' button URL
            resultsURL = $('.auctions-footer-button').attr('href').split('?')[0];

            if(value || watched) {
                resultsURL += '?';

                if(value) {
                    params.search = value;
                }

                if(watched) {
                    params.watched = watched ? 1 : 0;
                }

                resultsURL += $.param(params, true);
            }

            $('.auctions-footer-button').attr('href', resultsURL);

            // Get the auction UI elements
            $current = $('.post-type-archive-bat-listing .auctions-current:not(.auctions-current-sorter)');
            $auctions = $current.find('.auctions-item-container:not(.auctions-item-container-sorter)');

            if (0 === terms.length && false === watched) {
                $auctions.removeClass('auctions-item-container-hidden');

                $('.post-type-archive-bat-listing #auctions_filtered_message_none').hide();
                $('.post-type-archive-bat-listing #auctions_filtered_message_more').hide();
            } else {
                $filtered = $auctions.filter(function (index, element) {
                    var $element = $(element), searchable = $element.data('searchable'), category, comparer, keyword, found, term;

                    for (var i = 0; i < terms.length; i++) {
                        term = jQuery.trim(terms[i].toLowerCase());
                        found = false;

                        comparer = function (stringToCompare) {
                            var compare = stringToCompare.toString().toLowerCase(),
                                regex = new RegExp('(^|\\s)+' + term);

                            return compare.match(regex);
                        };

                        for (var j = 0; j < searchable.category.length; j++) {
                            var category = searchable.category[j];

                            if (comparer(category)) { found = true; break; }
                        }

                        for (var k = 0; k < searchable.keyword.length; k++) {
                            var keyword = searchable.keyword[k];

                            if (comparer(keyword)) { found = true; break; }
                        }

                        if (comparer(searchable.title)) { found = true; }
                        if (comparer(searchable.year)) { found = true; }
                        if (comparer(searchable.make)) { found = true; }
                        if (comparer(searchable.model)) { found = true; }
                        if (comparer(searchable.vin)) { found = true; }

                        if (found) { continue; }

                        return false;
                    }

                    return false === watched || !!$element.data('watched');
                });

                $filtered.removeClass('auctions-item-container-hidden');
                $auctions.not($filtered).addClass('auctions-item-container-hidden');

                $('.post-type-archive-bat-listing #auctions_filtered_message_none_terms').text(value);
                $('.post-type-archive-bat-listing #auctions_filtered_message_more_count').text($auctions.length - $filtered.length);
                $('.post-type-archive-bat-listing #auctions_filtered_message_none_watched').toggle(watched);
                $('.post-type-archive-bat-listing #auctions_filter_message_none_terms_exist').toggle('' !== value);
                $('.post-type-archive-bat-listing #auctions_filtered_message_none').toggle($filtered.length === 0);
                $('.post-type-archive-bat-listing #auctions_filtered_message_more').toggle(true);
            }

            $('.post-type-archive-bat-listing .toolbar-search-input').parents('.toolbar-search-container').toggleClass('toolbar-search-container-working', false);

            // On window unload POST the terms - only register this once
            if (false === registered) {
                window.onbeforeunload = function (e) {
                    if (filtered.length) {
                        $.ajax({
                            async: false,
                            data: { action: BaT_Theme.ajaxActionTrackFilteredTerms, terms: filtered, nonce: BaT_Theme.ajaxTrackFilteredTermsNonce },
                            method: 'POST',
                            url: BaT_Theme.ajaxUrl
                        });
                    }

                    return undefined;
                };

                registered = true;
            }
        };

        $(document).on('keyup', '.post-type-archive-bat-listing .toolbar-search-input', function (event) {
            if (null !== handle) { clearTimeout(handle); handle = null; }

            $(this).parents('.toolbar-search-container').toggleClass('toolbar-search-container-clearable', '' !== $.trim($(this).val()));
            $(this).parents('.toolbar-search-container').toggleClass('toolbar-search-container-working', true);


            $('.post-type-archive-bat-listing .toolbar-search-input').not($(this)).val($(this).val());

            handle = setTimeout(filter, 1000);
        });

        $(document).on('click', '.post-type-archive-bat-listing .toolbar-button-watched', function (event) {
            event.preventDefault();

            var $button = $(this);

            $button.toggleClass('toolbar-button-active').data('watched', 0 === $button.data('watched') ? 1 : 0);

            filter();
        });

        $(document).on('click', '.post-type-archive-bat-listing .toolbar-search-clear', function (event) {
            event.preventDefault();

            if (null !== handle) { clearTimeout(handle); handle = null; }

            $('.post-type-archive-bat-listing .toolbar-search-input').val('');
            $('.post-type-archive-bat-listing .toolbar-search-input').parents('.toolbar-search-container').toggleClass('toolbar-search-container-clearable', '' !== $.trim($('.toolbar-search-input').val()));

            if ($(this).is('.toolbar-search-clear-all')) {
                var $button = $('.post-type-archive-bat-listing .toolbar-button-watched');

                $button.removeClass('toolbar-button-active').data('watched', 0);
            }

            filter();
        });

        return {
            filter: filter
        };
    }());

    // Listings overview page - inline pagination
    (function () {
        $(document).on('click', '.auctions-footer-previous .auctions-footer-button', function (event) {
            event.preventDefault();

            var $this = $(this),
                $load = $this.siblings('.auctions-footer-loading').addClass('auctions-footer-loading-display'),
                $auctionsO = $this.parents('.auctions-footer-previous').siblings('.auctions-previous');
            $paginationO = $this.parents('.auctions-footer-previous'),
                href = $this.attr('href'),
                selector = $this.data('selector');

            // Remove the button so it can't get clicked multiple times
            $this.remove();

            $.get(
                href, {}, function (data, status) {
                    var $auctionsN = selector ? $(data).find(selector) : $(data).find('.auctions-previous'),
                        $auctionsNItems = $auctionsN.find('.auctions-item-container'),
                        $paginationN = $auctionsN.siblings('.auctions-footer-previous');

                    $auctionsO.append($auctionsNItems);
                    $paginationO.replaceWith($paginationN);
                }
            );
        });
    }());

    // No Reserve page
    (function () {
        $('.auctions-no-reserve .auctions-item-title').each(function(index, element) {
            new Dotdotdot(element, {
                height: "watch", // Update height on resize for @media rules
            });
        });
    }());

    // Listing bid box "Ends On" in local time - localize based on system timezone
    (function () {
        var $ends = $('[data-ends]'),
            timestamp = $ends.data('ends') * 1000,
            datetime = new Date(timestamp),
            datetimem = moment(datetime),
            formatted = datetimem.format('dddd, MMMM D [at] h:mma');

        $ends.text(formatted);
    }());

    // hack for fixing listing sticky position on input focus
    (function () {
        if ('ontouchstart' in window) {
            $(document).on('focus', 'textarea, input, select', function () {
                $('.listing-sticky').css({
                    visibility: 'hidden'
                });
            }).on('blur', 'textarea,input,select', function () {
                $('.listing-sticky').css({
                    visibility: ''
                });
            });
        }
    }());

    // Listing header button click functionality
    // When clicking a listing header button, smooth scroll to the desired element
    (function () {
        var $sticky = $('.listing-sticky');

        $(document).on('click', '.listing-available-actions .button', function (event) {
            var href = $(this).attr('href');

            if (0 === href.indexOf('#')) {
                var $element = $(href);

                if ($element.size() > 0) {
                    event.preventDefault();

                    $('html, body').animate({
                        scrollTop: ($element.offset().top - $sticky.height())
                    }, 250);
                }
            }
        });
    }());

    // Listing header watching button toggle
    (function () {
        BaT_Theme.toggleWatchingStatus = function () {
            $('.loggedin .button-watching, .loggedin .link-watching').each(function (index, element) {
                var $link = $(element),
                    $toggle = $link.find($link.data('toggle-target')),
                    $text = $link.find($link.data('text-target')),
                    toggle = $link.data('toggle'),
                    text = $link.data('text');

                $link.data('text', $text.text());

                $toggle.toggleClass(toggle);
                $text.text(text);
            });
        };

        $('.loggedin .button-watching, .loggedin .link-watching').click(function (event) {
            event.preventDefault();

            BaT_Theme.toggleWatchingStatus();

            $.post(
                $(this).attr('href'),
                { ajax: true },
                function (data, status) {
                    // What should happen here?
                },
                'json'
            );
        })

        $('#poppable-link-reminder').click(function () {
            if (!isWatchingListing()) {
                BaT_Theme.toggleWatchingStatus();
                $.post(
                    $(this).data('toggle-watch-url'),
                    { ajax: true }
                );
            }
        });

        function isWatchingListing() {
            return $(".button-watching .button-watch-icon-watching").length > 0;
        }
    }());

    // Listing header sticky portion scroll down from top
    (function () {
        var $header = $('.site-header'),
            $sticky = $('.listing-sticky'),
            $title = $sticky.find('.listing-post-title'),
            $wrap = $('<div></div>').height($sticky.height()),
            height = $(window).width() > 750 ? $header.outerHeight() : $header.outerHeight() + $title.outerHeight();

        $sticky.wrap($wrap);

        if ($sticky.length > 0) {
            $(document).scroll(function () {
                var $document = $(document),
                    scrolled = $document.scrollTop();

                if (scrolled >= height && !$sticky.is('.listing-sticky-fixed')) {
                    $sticky.addClass('listing-sticky-fixed');
                } else if (scrolled < height && $sticky.is('.listing-sticky-fixed')) {
                    $sticky.removeClass('listing-sticky-fixed');
                }
            }).trigger('scroll');
        }
    }());

    // Listing header countdown and progress bar
    (function () {
        var $progress = $('.listing-available-progress-bar'),
            $countdown = $('.listing-available-countdown'),
            $notice = $countdown.find('.listing-end-processing-notice'),
            layout = '{d<}{dn} {dl}, {d>} {h<}{hn}:{h>}{m<}{mnn}:{m>}{snn}',
            expiration = function () { },
            progression = function (parts) { },
            until = new Date($countdown.data('until') * 1000);

        expiration = function () {
            if ($countdown.siblings('.listing-end-processing-notice').size()) {
                $countdown.siblings('.listing-end-processing-notice').show();
                $countdown.hide();
            }
        };

        progression = function (parts) {
            var seconds = jQuery.countdown.periodsToSeconds(parts),
                extension = parseFloat(BaT_Theme.extensionWindowSeconds),
                width = ((Math.min(extension, seconds) / extension) * 100) + '%';

            $progress.toggleClass('listing-available-progress-bar-counting', seconds <= extension)
                .toggleClass('listing-available-progress-bar-finalmin', seconds <= 60)
                .css('width', width);
        };

        $countdown.countdown({
            layout: layout,
            onExpiry: expiration,
            onTick: BaT_Theme.getTickableFunction($countdown, progression),
            until: until
        });
    }());

    // Links that do various actions
    (function () {
        $(document).on('click', '.loggedin .ajax-link', function (event) {
            event.preventDefault();

            var $link = $(this);

            $.post(
                $link.attr('href'),
                { ajax: true },
                function (data, status) { },
                'json'
            );
        });

        $(document).on('click', '.logged-in .ajax-toggle-link', function (event) {
            event.preventDefault();

            var $link = $(this),
                ajaxLinkAlt = $link.data('toggle-ajax-link-alt'),
                altHref,
                href;

            href = $link.attr('href');

            if (ajaxLinkAlt) {
                altHref = $link.data('toggle-ajax-link-alt');
                $link.data('toggle-ajax-link-alt', href);
                $link.attr('href', altHref);
            }
        });

        $(document).on('click', '.logged-in .class-toggle-link', function (event) {
            event.preventDefault();

            var $link = $(this),
                classes = $link.data('toggle-class-classes').toString().split('|'),
                selectors = $link.data('toggle-class-selectors').toString().split('|');

            if (classes.length === selectors.length) {
                for (var i = 0; i < selectors.length; i++) {
                    var klass = classes[i], selector = selectors[i];

                    $(selector).toggleClass(klass);
                }
            }
        });

        $(document).on('click', '.logged-in .phrase-toggle-link', function (event) {
            event.preventDefault();

            var $link = $(this), $element, $toggles,
                phrase = $link.data('toggle-text-phrase'),
                phraseAlt = $link.data('toggle-text-phrase-alt'),
                selector = $link.data('toggle-text-selector');

            $elements = $(selector);
            $toggles = $('[data-toggle-text-selector="' + selector + '"]');

            if (phraseAlt) {
                $toggles.data('toggle-text-phrase', phraseAlt).data('toggle-text-phrase-alt', false);
            } else {
                $toggles.data('toggle-text-phrase', $elements.first().text());
            }

            $elements.text(phrase);
        });
    }());

    // Listings preview page - paginate listings
    (function () {
        var paginate = function (event) {
            if (37 === event.which) {
                $('#modal-listing .modal-arrow-prev').trigger('click');
            } else if (39 === event.which) {
                $('#modal-listing .modal-arrow-next').trigger('click');
            }
        };

        $('#modal-listing').on('bat.modal.show', function () {
            $(document).one('keydown.modal-listing', paginate);
        });

        $('#modal-listing').on('bat.modal.hide', function () {
            var $this = $(this),
                $modalContent = $this.find('.modal-content');

            BaT_Theme.player.pauseChildren($modalContent);

            $(document).off('keydown.modal-listing');
        });

        $(document).on('click', '#modal-listing .modal-arrow, .listing-feature-modal-paginator > .item', function (event) {
            event.preventDefault();
            event.stopPropagation();
            event.stopImmediatePropagation();

            var $control = $(this),
                $modalContent = $('#modal-listing .modal-content'),
                $listingFeatureContent = $('#modal-listing .listing-feature-modal-content'),
                $feature = $('.listing-feature[data-id="' + $listingFeatureContent.data('id') + '"]'),
                $paged;

            if ($control.is('.modal-arrow-prev,.item-prev')) {
                $paged = $feature.prevAll('.listing-feature.listing-feature-modaled').first();

                if (0 === $paged.length) {
                    $paged = $('.listing-feature.listing-feature-modaled').last();
                }
            } else if ($control.is('.modal-arrow-next,.item-next')) {
                $paged = $feature.nextAll('.listing-feature.listing-feature-modaled').first();

                if (0 === $paged.length) {
                    $paged = $('.listing-feature.listing-feature-modaled').first();
                }
            }

            if (0 !== $paged.length) {
                BaT_Theme.player.pauseChildren($modalContent);
                $paged.find('.modal-link').first().trigger('click');
            }
        });

        if ($('.listing-feature-modaled').length <= 1) {
            $('#modal-listing > .modal-arrow').hide();
        }
    }());

    // Listings preview page - form validation
    (function () {
        $(document).on('submit', '.submission-modal-form', function (event) {
            var $form = $(this),
                $inputs = $form.find(':input'),
                $invalid;

            $inputs.parents('.form-field').removeClass('form-field-error');

            $invalid = $inputs.filter(function (index, input) {
                var $input = $(input),
                    email = $input.is('#bat_submission_email'),
                    login = $input.is('#bat_submission_login'),
                    phone = $input.is('#bat_submission_phone'),
                    value = $.trim($input.val());

                if (email) {
                    return '' === value || -1 === value.indexOf('@');
                } else if (login) {
                    return '' === value || value.length < 3 || -1 !== value.indexOf('@') || -1 !== value.indexOf(' ');
                } else if (phone) {
                    return '' === value || '' === value.replace(/[^\d]/g, '');
                } else {
                    return '' === value;
                }
            });

            if ($invalid.length) {
                event.preventDefault();

                $invalid.parents('.form-field').addClass('form-field-error');
                $invalid.first().focus();
            }
        });
    }());

    // Listings preview page - form year / make / model
    (function () {
        var $form = $('.submission-modal-form'),
            $year = $form.find('#year'),
            $make = $form.find('#make'),
            $model = $form.find('#model'),
            makes = $year.data('makes') || [];

        if (0 === $form.length || 0 === $year.length || 0 === $make.length || 0 === $model.length) { return; }

        $year.change(function (event) {
            var make = $make.val(),
                model = $model.val(),
                year = $year.val(),
                data = 'undefined' === typeof makes[year] ? {} : makes[year],
                exists = false;

            $make.find('option').filter(function (index, element) {
                var $element = $(element);

                return '' !== $element.attr('value');
            }).remove();

            for (var p in data) {
                if (data.hasOwnProperty(p)) {
                    exists = true;

                    $make.append($('<option>').attr('value', p).text(p));
                }
            }

            if ($make.find('option').filter(function (index, element) { return make === $(element).attr('value'); }).length > 0) {
                $make.val(make);
            }

            $make.trigger('change').prop('disabled', '' === year);
        }).trigger('change');

        $make.change(function (event) {
            var make = $make.val(),
                model = $model.val(),
                year = $year.val(),
                data = 'undefined' === typeof makes[year] || 'undefined' === typeof makes[year][make] ? [] : makes[year][make],
                exists = data.length > 0;

            $model.find('option').filter(function (index, element) {
                var $element = $(element);

                return '' !== $element.attr('value');
            }).remove();

            for (var i = 0; i < data.length; i++) {
                $model.append($('<option>').attr('value', data[i]).text(data[i]));
            }

            if ($model.find('option').filter(function (index, element) { return model === $(element).attr('value'); }).length > 0) {
                $model.val(model);
            } else {
                $model.val($model.find('option:visible').first().attr('value'));
            }

            $model.trigger('change').prop('disabled', '' === make);
        });
    }());

    // Listings preview page - inquiry
    (function () {
        $(document).on('click', '.button-listing-inquiry', function (event) {
            event.preventDefault();

            var $button = $(this),
                $fields = $button.parents('.modal-popup').find('.form-fields'),
                $message = $button.parents('.modal-popup').find('.message-listing-inquiry'),
                $text = $fields.find('[name="bat_text"]');

            $.post(
                BaT_Theme.ajaxUrl,
                {
                    action: BaT_Theme.ajaxActionSellerMessage,
                    ajax: true,
                    item: $button.data('id'),
                    text: $.trim($text.val())
                },
                function (data, status) { },
                'json'
            );

            $button.hide();
            $fields.hide();
            $message.show();
            $text.val('');
        });

        $(document).on('click', '.button-listing-inquiry-back', function (event) {
            event.preventDefault();

            var $button = $(this).siblings(),
                $fields = $button.parents('.modal-popup').find('.form-fields'),
                $message = $button.parents('.modal-popup').find('.message-listing-inquiry');

            $button.show();
            $fields.show();
            $message.hide();
        });
    }());

    // Same page scrollable links
    (function () {
        $('.scrollable-link').click(function(event) {
            var href = $(this).attr('href'),
                time = $(this).data('time') || 250,
                $element = $(href),
                $sticky = $('.listing-sticky');

            if(0 === href.indexOf('#') && $element.length > 0) {
                event.preventDefault();

                $('html, body').animate({
                    scrollTop: ($element.offset().top - $sticky.height())
                }, time);
            }
        });
    }());

    // Modal - new style
    (function () {
        var BAT = window.BAT = window.BAT || {};

        BAT.modal = {
            hide: function () {
                var $document = $(document), top = $document.data('top');

                $('html,body').removeClass('modal-shown').css('top', '');
                $document.scrollTop(top).data('top', false);
                $('.modal-cover-shown').removeClass('modal-cover-shown').each(function (index, element) {
                    $(element).trigger('bat.modal.hide');
                });

                $(document).off('keydown.bat_modal');

                if (window.history && window.history.pushState && BaT_Theme.currentUrl) {
                    window.history.go(-1 * window.history.length);
                    window.history.replaceState({ modal: { element: false } }, "", BaT_Theme.currentUrl);
                }

                BAT.modal.shown = false;
            },

            show: function (selector, content, viewModel) {
                var $element = $(selector);

                if ($element.is('.modal-cover')) {
                    var $document = $(document), $modals = $('.modal-cover'), top = $document.data('top') || $document.scrollTop();

                    if (content) {
                        $element.find('.modal-content').html(content);
                    }

                    if (viewModel) {
                        // Allow rebinding Knockout bindings in modals without losing jQuery bindings (https://stackoverflow.com/a/47388645)
                        var cleanExternalDataOriginal = ko.utils.domNodeDisposal.cleanExternalData;
                        ko.utils.domNodeDisposal.cleanExternalData = function () {};
                        ko.cleanNode($element[0]);
                        ko.utils.domNodeDisposal.cleanExternalData = cleanExternalDataOriginal;

                        ko.applyBindings(viewModel, $element[0]);
                    }

                    $('html,body').addClass('modal-shown');
                    $('body').css('top', -1 * top);
                    $element.addClass('modal-cover-shown');
                    $document.data('top', top);

                    BaT_Theme.featuredListingsSliders();
                    BaT_Theme.featuredListingsOEmbeds();

                    $modals.not($element).removeClass('modal-cover-shown').each(function (index, element) {
                        $(element).trigger('bat.modal.hide');
                    });

                    setTimeout(function () { $(window).trigger('resize'); }, 0);

                    $(document).on('keydown.bat_modal', function (event) {
                        if ('Escape' === event.key) {
                            BAT.modal.hide();
                        }
                    });

                    $element.trigger('bat.modal.show');

                    BAT.modal.shown = true;
                }

                return $element;
            },

            shown: false,

            // Useful defaults for the jQuery validator plugin for use in modals
            validatorDefaults: {
                errorClass: 'control-group-error',
                errorPlacement: function(error, element) {
                    $(error).css('width', '100%');
                    error.insertAfter($(element).closest('label'))
                },
                errorContainer: '.error-summary',
            }
        };

        $(document).on('click', '.modal-close, .modal-cover', function (event) {
            if ($(event.target).is('.modal-close, .modal-cover')) {
                event.preventDefault();
                event.stopPropagation();

                BAT.modal.hide();
            }
        });

        $(document).on('click', '.modal-link, .loggedin .modal-link-loggedin', function (event) {
            event.preventDefault();

            var $link = $(this), $modal, content, viewModel, identifier, selector;

            if ($link.data('selector')) {
                selector = $link.data('selector');
            } else {
                selector = $link.attr('href');
            }

            if ($link.data('content')) {
                content = $($link.data('content')).html();
            } else {
                content = false;
            }

            if ($link.data('view-model')) {
                viewModel = $link.data('view-model');
            } else {
                viewModel = false;
            }

            if (window.history && window.history.pushState) {
                if ($link.attr('id')) {
                    identifier = $link.attr('id');
                } else {
                    identifier = BaT_Theme.guid();

                    $link.attr('id', identifier);
                }

                if (BAT.modal.shown) {
                    window.history.replaceState({ modal: { element: identifier } }, "", $link.attr('href'));
                } else {
                    window.history.pushState({ modal: { element: identifier } }, "", $link.attr('href'));
                }
            }

            $modal = BAT.modal.show(selector, content, viewModel);
        });

        setTimeout(function () {
            $('.modal-link[data-force="1"], .loggedin .modal-link-loggedin[data-force="1"]').first().trigger('click');
        }, 0);

        window.onpopstate = function (event) {
            if (null === event.state && BAT.modal.shown) {
                BAT.modal.hide();
            } else if (event.state && event.state.modal) {
                if (false === event.state.modal.element) {
                    BAT.modal.hide();
                } else {
                    $('#' + event.state.modal.element).click();
                }
            }
        };
    }());

    // Modal bid handling
    BaT_Theme.bid = (function() {
        // Add custom KO bindings for displaying USD amounts like "$5,000.00" with currencyText: binding.
        // Use along with currencyDecimals: binding to control the number of decimal places.
        ko.bindingHandlers.currencyText = {
            update: function(element, valueAccessor, allBindings) {
                var value = ko.utils.unwrapObservable(valueAccessor()),
                    decimals = allBindings.get('currencyDecimals'),
                    formattedValue = BaT_Theme.bid.formatCurrency(value, {decimals: decimals});

                ko.bindingHandlers.text.update(element, function() { return formattedValue; });
            },
        };

        $('#bat-listings-place-bid').click(function(event) {
            var $placeBidButton = $(this),
                $bidAmountInput = $('#bat-listings-listings-amount'),
                $bidRow = $('[data-listing-current-bidding="' + BaT_Theme.post + '"]'),
                bidAmount = BaT_Theme.bid.sanitizeAmount($bidAmountInput.val()),
                highBidAmount = $bidRow.data('amount') ? $bidRow.data('amount') : 0,
                highBidUser = $bidRow.data('high-bidder') ? $bidRow.data('high-bidder') : false,
                minimumBid = $bidRow.data('minimum'),
                maximumBid = $bidRow.data('maximum'),
                currentTime = (new Date).getTime(),
                listingEndsTime = $('.listing-available-countdown').countdown('option', 'until').getTime(),
                isFfBid = BaT_Theme.bid.isFfBid(bidAmount, highBidAmount, listingEndsTime),
                isFfVerified = $placeBidButton.data('ff-verified') === true,
                isMoreThanFiveTimesBidOnLastDay = highBidAmount > 0 && ((listingEndsTime - currentTime) < 86400000 /* ms in a day */) && (bidAmount > 5 * highBidAmount);

            if(highBidUser && highBidUser == BAT_VMS.userId) {
                $placeBidButton.data('selector', '#modal-bid-error-generic');
                var viewModel = {
                    errorTitle: BaT_Theme.bidModalText.alreadyWinningTitle,
                    errorText: BaT_Theme.bidModalText.alreadyWinningMessage,
                };

                $placeBidButton.data('view-model', viewModel)
            } else if(bidAmount < minimumBid) {
                $placeBidButton.data('selector', '#modal-bid-error-generic');
                var viewModel = {
                    errorTitle: BaT_Theme.bidModalText.bidTooLowTitle,
                    errorText: $.validator.format(BaT_Theme.bidModalText.bidTooLowMessage, BaT_Theme.bid.formatCurrency(minimumBid, {decimals: 0})),
                };

                $placeBidButton.data('view-model', viewModel);
            } else if(bidAmount > maximumBid) {
                $placeBidButton.data('selector', '#modal-bid-error-generic');
                var viewModel = {
                    errorTitle: BaT_Theme.bidModalText.bidExceedsMaximumTitle,
                    errorText: BaT_Theme.bidModalText.bidExceedsMaximumMessage,
                };

                $placeBidButton.data('view-model', viewModel);
            } else if (isMoreThanFiveTimesBidOnLastDay) {
                $placeBidButton.data('selector', '#modal-bid-error-generic');
                var viewModel = {
                    errorTitle: BaT_Theme.bidModalText.bidExceedsMaximumOnLastDayTitle,
                    errorText: BaT_Theme.bidModalText.bidExceedsMaximumOnLastDayMessage,
                };

                $placeBidButton.data('view-model', viewModel);
            } else if (isFfBid && !isFfVerified) {
                $placeBidButton.data('selector', '#modal-bid-ff-confirmation');

                var viewModel = {
                    bidAmount: bidAmount,
                    highBidAmount: highBidAmount,
                };

                $placeBidButton.data('view-model', viewModel);
            } else {
                $placeBidButton.data('selector', '#modal-bid-confirmation');
                var viewModel = {
                    bidAmount: bidAmount,
                    isFfVerified: isFfVerified,
                    serviceFee: BaT_Theme.bid.calculateServiceFee(bidAmount),
                    userHasBid: BaT_Theme.bid.userHasBid(),
                };

                $placeBidButton.data('view-model', viewModel);

                // Clear any page bidding state after a delay
                setTimeout(function () {
                    $('#bat-listings-listings-amount').val('');
                }, 1000);
            }
        });

        $('#bat-listings-listings-amount').keydown(function(event) {
            // Open bidding modal on enter key in amount input
            if(event.keyCode == 13) {
                $('#bat-listings-place-bid').click();
            }
        });

        var calculateServiceFee = function(bidAmount) {
            var buyerFee = parseInt(bidAmount, 10) * BaT_Theme.bidModalBuyerFeePercentage;

            if(buyerFee > BaT_Theme.bidModalBuyerFeeMaximum) {
                buyerFee = BaT_Theme.bidModalBuyerFeeMaximum;
            } else if(buyerFee < BaT_Theme.bidModalBuyerFeeMinimum) {
                buyerFee = BaT_Theme.bidModalBuyerFeeMinimum;
            }

            return buyerFee;
        };

        var formatCurrency = function(value, options) {
            var decimals = (options.decimals === undefined) ? 2 : options.decimals;
                valueWithDecimals = Number(value).toFixed(decimals),
                valueWithCommas = valueWithDecimals.toString().replace(/\B(?=(\d{3})+(?!\d))/g, ',');
            return valueWithCommas;
        }

        var handleRestError = function(error) {
            var errorMessage = error.message,
                errorTitle = error.data.error_title
                errorModal = '#modal-bid-error-generic';

            if(error.code === 'authorization_failed') {
                errorModal = '#modal-bid-error-authorization';
            } else if(error.code === 'rest_cookie_invalid_nonce') {
                errorTitle = BaT_Theme.bidModalText.restNonceErrorTitle;
                errorMessage = BaT_Theme.bidModalText.restNonceErrorMessage;
            } else if(error.code === 'requires_stripe_action') {
                $form = $('.modal-cover-shown form');
                return BaT_Theme.bid.handleStripeAction($form, error.data.payment_intent_client_secret)
            }

            var viewModel = {
                errorTitle: errorTitle,
                errorText: errorMessage,
            };

            BAT.modal.show(errorModal, /* content */ false, viewModel);
        }

        var handleStripeAction = function($form, clientSecret) {
            // Stripe's 3D-Secure dialog adds an inset to the page which overrides the bidding modal's fixed positioning
            // and causes a conflict resulting in a black background. To prevent this, add a temporary !imporant override
            // class to keep the bidding modal's positioning fixed.
            var $body = $('body'),
                top = $body.css('top');
            $('<style type="text/css"> .modal-3dsecure-override{ inset: initial !important; top: ' + top + ' !important; } </style>').appendTo('head');
            $body.addClass('modal-3dsecure-override');

            // Trigger 3D-Secure dialog if required
            BaT_Listings_Stripe.stripe.handleCardAction(
                clientSecret
            ).then(function(result) {
                $body.removeClass('modal-3dsecure-override');

                if (result.error) {
                    $form.find('input[name="payment_intent_id"]').remove();
                    $form.find('input[type="submit"]').removeProp('disabled');

                    var viewModel = {
                        errorTitle: BaT_Theme.bidModal3DSecureFailureTitle,
                        errorHtml: BaT_Theme.bidModal3DSecureFailureHtml,
                    };

                    BAT.modal.show('#modal-bid-error-generic', /* content */ false, viewModel);
                } else {
                    // Resubmit the form with the approved payment intent
                    $('<input type="hidden" name="payment_intent_id" />')
                        .val(result.paymentIntent.id)
                        .appendTo($form);

                    $form.submit();
                }
            });
        };

        var isFfBid = function(bidAmount, highBidAmount, listingEndsTime) {
            var currentTime = (new Date).getTime(),
                isInFfWindow = (currentTime + BaT_Theme.bidModalFFWindowInMinutes * 60000 /* ms in a minute */) > listingEndsTime,
                minFfBid = false;

            if(!isInFfWindow || highBidAmount === 0) {
                return false;
            }

            if (highBidAmount > 100000) {
                minFfBid = 1.2 * highBidAmount;
            } else if (highBidAmount > 50000) {
                minFfBid = 1.5 * highBidAmount;
            } else if (highBidAmount > 20000) {
                minFfBid = 1.75 * highBidAmount;
            } else if (highBidAmount > 10000) {
                minFfBid = 2.0 * highBidAmount;
            } else if (highBidAmount > 5000) {
                minFfBid = 3.0 * highBidAmount;
            } else if (highBidAmount > 1000) {
                minFfBid = 4.0 * highBidAmount;
            }

            if(minFfBid && bidAmount > minFfBid) {
                return true;
            } else {
                return false;
            }
        }

        var sanitizeAmount = function(bidAmount) {
            bidAmount = bidAmount.replace(/[^\d\.\,]/, '');
            bidAmount = bidAmount.replace(/(\.|,)\d{0,2}$/, '');
            bidAmount = bidAmount.replace(/[^\d]/, '');
            bidAmount = parseInt(bidAmount, 10 /* radix */);

            return bidAmount || 0;
        }

        var userHasBid = function() {
            // Use comment data to determine if the user has created a bid on this page
            if(!BAT_VMS.userId) { return false; }

            // Check bids in BAT_VMS.comments from initial page load
            var initialPageDataBids        = BAT_VMS.comments.filter(function(comment) { return comment.type == 'bat-bid' && comment.authorId == BAT_VMS.userId; });
            var isUserBidInInitialPageData = initialPageDataBids.length > 0;

            // Also check in BAT_VMS.CLVM.comments, which will include bids loaded dynamically since page load
            var loadedPageDataBids = BAT_VMS.CLVM.comments().filter(function(comment) { return comment.type() == 'bat-bid' && comment.authorId() == BAT_VMS.userId; });
            var isUserInLoadedPageDataBids = loadedPageDataBids.length > 0;

            return isUserBidInInitialPageData || isUserInLoadedPageDataBids;
        }

        return {
            calculateServiceFee: calculateServiceFee,
            handleRestError: handleRestError,
            handleStripeAction: handleStripeAction,
            formatCurrency: formatCurrency,
            isFfBid: isFfBid,
            sanitizeAmount: sanitizeAmount,
            userHasBid: userHasBid,
        }
    })();

    // Model page subscription button
    (function () {
        $('#button-notifications').click(function (event) {
            event.preventDefault();

            var $this = $(this), handle = $this.data('handle');

            handle = 'undefined' === typeof handle ? null : handle;

            if (null !== handle) {
                clearTimeout(handle);
                $this.data('handle', null);
            }

            if (!$this.hasClass('poppable-link')) {
                $this.data('watched', 'yes' === $this.data('watched') ? 'no' : 'yes');
                $this.text('yes' === $this.data('watched') ? $this.data('toggled-on') : $this.data('toggled-off'));

                handle = setTimeout(function() {
                    $.post(
                        $this.attr('href'),
                        {
                            ajax: true,
                            force: $this.data('watched')
                        },
                        function (data, status) {

                        },
                        'json'
                    );
                }, 750);

                $this.data('handle', handle);
            }
        });
    }());

    // Subscribe to Model from the Make Page -- subscription button
    (function () {
        $('.button-model-notification--toggle').click(function (event) {
            event.preventDefault();

            var $this = $(this), handle = $this.data('handle');

            handle = 'undefined' === typeof handle ? null : handle;

            if (null !== handle) {
                clearTimeout(handle);
                $this.data('handle', null);
            }

            if (!$this.hasClass('poppable-link')) {
                $this.toggleClass('model--subscribed');
                $this.data('watched', 'yes' === $this.data('watched') ? 'no' : 'yes');

                // Debounce the network request
                handle = setTimeout(function() {
                    var $text = $this.find('.button-text'),
                        $watch = $this.data('watched');

                    $.post(
                        $this.attr('href'),
                        {
                            ajax: true,
                            force: $this.data('watched')
                        },
                        function (data) {

                        },
                        'json'
                    ).fail(function() {
                        $this.toggleClass('model--subscribed');
                    });
                }, 750);

                $this.data('handle', handle);
            }
        });
    }());

    // General non-link clickable elements
    (function () {
        $(document).on('click', '[data-clickable-element]', function (event) {
            event.preventDefault();

            document.location.href = $(this).data('clickable-element');
        });
    }());

    // Checkbox table
    (function () {
        $(document).on('change', '.bat-col-checkbox input[type="checkbox"]', function (event) {
            var $this = $(this),
                index = $this.parents('th').index();

            $this.parents('table').find('tbody tr').find('td:eq(' + index + ') input[type="checkbox"]').prop('checked', $this.prop('checked'));
        });
    }());

    // Image voting
    (function () {
        $(document).on('dblclick', '.image-vote-container', function (event) {
            $(this).find('.image-vote a').click();
        }).on('click', '.image-vote a, .pswp__button--votes a', function (event) {
            var $this = $(this),
                $span = $('<span class="comment-actions-approve"></span>').html($this.html()),
                item_id = $this.find('[data-votes-id]').data('votes-id'),
                url = $this.is('a') ? $this.attr('href') : $this.data('href');

            if ($this.parents('body.logged-in').length > 0) {
                event.preventDefault();

                $.get(
                    url,
                    { ajax: true },
                    function (data, status) { },
                    'json'
                );

                $('[data-votes-id="' + item_id + '"]').parents('a, button').replaceWith($span.get(0).outerHTML);

                for (var i = 0; i < window.galleryItems.length; i++) {
                    var item = window.galleryItems[i];

                    if (item.id == item_id) {
                        item.voted = true;
                    }
                }
            } else { /* do nothing because we want link clicks to propagate appropriately */ }
        });
    }());

    // Model threshold
    (function () {
        $('.previous-listings-more[data-threshold]').each(function (index, element) {
            var $element = $(element),
                $container = $element.prev('.previous-listings'),
                $items = $container.find('> *'),
                $button = $element.find('a.page-numbers'),
                threshold = $element.data('threshold');

            $items.filter(':gt(' + (threshold - 1) + ')').hide();

            $button.click(function (event) {
                event.preventDefault();

                $items.show();
                $element.remove();
            });
        })
    }());

    // Modal pop immediately
    (function () {
        var $pops = $('[data-pop-immediately]');

        if ($pops.length > 0) {
            $.magnificPopup.open({
                items: {
                    src: $pops.first().html(),
                    type: 'inline'
                }
            });
        }
    }());

    // Moderate comments
    (function () {
        $(document).on('click', '.moderate-comments-toggle', function (event) {
            event.preventDefault();

            var $button = $(this);

            $button.prop('disabled', true);

            $.post(
                BaT_Theme.ajaxUrl,
                {
                    action: BaT_Theme.ajaxActionToggleModerateComments,
                    item: BaT_Theme.post
                },
                function (data, status) { },
                'json'
            );
        });
    }());

    // Slideable cards
    (function() {
        var initializeSwiper = function($cardsContainer) {
            $cardsContainer.addClass('swiper-container');
            $cardsContainer.find('.bat-cards-wrapper').addClass('swiper-wrapper');
            $cardsContainer.find('.bat-card').addClass('swiper-slide');

            var swiper = new Swiper($cardsContainer.get(0), {
                centeredSlides: true,
                slidesPerView: 'auto',
                spaceBetween: 20,
                pagination: {
                    el: '.swiper-pagination',
                },
            });

            return swiper;
        };

        var destroySwiper = function($cardsContainer, swiper) {
            swiper.destroy();

            $cardsContainer.removeClass('swiper-container');
            $cardsContainer.find('.bat-cards-wrapper').removeClass('swiper-wrapper');
            $cardsContainer.find('.bat-card').removeClass('swiper-slide');
        };

        var optionSwiper = false,
            graphicSwiper = false;

        $(window).resize(function(event) {
            var $window = $(window),
                $cardsContainers = $('.bat-cards-container'),
                $optionContainer = $('#bat-submit-option-slider');
                $graphicContainer = $('#bat-submit-graphic-slider');
                widthCurrent = $window.outerWidth(),
                isMobile = widthCurrent < 751;

            $cardsContainers.toggleClass('bat-cards-container-mobile', isMobile);

            if($optionContainer.length && $graphicContainer.length) {
                if(isMobile && !optionSwiper && !graphicSwiper) {
                    optionSwiper = initializeSwiper($optionContainer);
                    graphicSwiper = initializeSwiper($graphicContainer);

                    // Set option and graphic sliders to slide together
                    optionSwiper.controller.control = graphicSwiper;
                    graphicSwiper.controller.control = optionSwiper;
                } else if(!isMobile && optionSwiper && graphicSwiper) {
                    destroySwiper($optionContainer, optionSwiper);
                    optionSwiper = false;

                    destroySwiper($graphicContainer, graphicSwiper);
                    graphicSwiper = false;
                }
            }
        }).trigger('resize');
    }());

    // Pills
    (function () {
        var $pills = $('.bat-pills');

        $pills.each(function (index, container) {
            var $container = $(container),
                $toolbar = $container.find('.bat-pills-toolbar'),
                $buttons = $toolbar.find('.bat-pill'),
                $contents = $container.find('.bat-pill-content').hide();

            $buttons.on('click', function (event) {
                event.preventDefault();

                var $button = $(this),
                    $content = $contents.filter($button.attr('href'));

                $buttons.not($button).removeClass('bat-pill-active');
                $contents.not($content).hide();

                $button.addClass('bat-pill-active');
                $content.show();
            }).filter('.bat-pill-active').first().trigger('click');
        });
    }());

    // Recent bids header thing
    BaT_Theme.recentBidsCountdowns();
    $(document).on('touchend', '.recent-bids-list-item a', function (event) {
        $(this).data('touched', true);
    }).on('mouseenter', '.recent-bids-list-item', function (event) {
        var $this = $(this);

        if (!$this.find('a').data('touched')) {
            $('.recent-bids-list-item').removeClass('recent-bids-list-item-active');

            $this.addClass('recent-bids-list-item-active');

            var bid = $(this).data('bid'),
                $image = $('.recent-bids-image-link[data-bid="' + bid + '"]').remove().prependTo('.recent-bids-image-links').show();
        }

    });

    // Open search on click
    $('#header-search-open, #header-search-close').click(function (event) {
        event.preventDefault();

        var $menu = $('.header-nav .header-menu'),
            $search = $('.header-nav .header-search');

        // If either of these are animating, let them finish
        if ($menu.is(':animated') || $search.is(':animated')) { return; }

        if ($menu.is(':visible')) {
            $menu.fadeOut(200, function () { $search.fadeIn(200, function () { $search.find('input').focus(); }); });
        } else if ($search.is(':visible')) {
            $search.fadeOut(200, function () { $menu.fadeIn(200); });
        }
    });

    // Mobile header slide
    $('.header-nav-toggle').click(function (event) {
        event.preventDefault();

        $('.header-nav').slideToggle();
    });

    // Mobile footer slide
    $('.footer-nav-toggle').click(function (event) {
        event.preventDefault();

        $('.footer-nav').slideToggle();
    });

    // Mobile category text
    if ($(window).width() < 452) {
        $('.toolbar-cats select #all-cats').text($('.toolbar-cats select #all-cats').data('cat-name'));
    }

    // When a user interacts with the comment box and they leave a comment that looks like a bid,
    // a popup appears that the user can dismiss by indicating they indeed did mean to leave a comment.
    $(document).on('click', '.cancel-comment', function (event) {
        event.preventDefault();

        $.magnificPopup.close();
    }).on('click', '.continue-comment', function (event) {
        event.preventDefault();

        $.magnificPopup.close();
        $('#comments-form').data('confirmed-as-comment', true).submit();
    });

    // When a comment is submitted, do a few things:
    // 1. Check to see if it looks like a bid and hasn't been previously confirmed as not a bid -
    // if it looks like a bid, pop up a modal
    // 2. Submit the comment and wait for the response - when response arrives, enable comment submission again
    $('.comments-form-container-logged-in #comments-form').submit(function (event) {
        var $this = $(this),
            $textarea = $this.find('textarea'),
            $comments = $this.parents('#comments');

        event.preventDefault();
        if ($.trim($textarea.val()).match(/^[$,\d\.]+$/) && !$this.data('confirmed-as-comment')) {
            $.magnificPopup.open({
                items: {
                    src: $('#comment-confirmation-modal-container').html(),
                    type: 'inline'
                }
            });
        } else {
            $textarea.prop('readonly', true);
            $this.ajaxSubmit({
                data: { ajax_submission: 'true' },

                dataType: 'json',

                beforeSubmit: function (arr, $form, options) {
                    $form.find('input[type="submit"]').prop('disabled', true).val('Please wait...');
                },
                error: function (xhr, status, text) {
                    $this.get(0).submit();
                    $textarea.prop('readonly', false);
                },
                success: function (data, status, xhr, $form) {
                    $form.data('confirmed-as-comment', false);
                    $form.find('textarea').val('').prop('readonly', false);
                    $form.find('input[type="submit"]').prop('disabled', false).val('Submit');

                    if ('undefined' !== typeof _gaq) {
                        _gaq.push(['_trackEvent', 'comment-posted', 'comment-posted']);
                    }

                    if (0 === parseInt(data.comment.approved) && true === data.first) {
                        // Add it to the view model
                        var CLVM = 'undefined' !== typeof BAT_VMS && 'undefined' !== typeof BAT_VMS.CLVM ? BAT_VMS.CLVM : false;

                        if (CLVM) { CLVM.addComment(data.comment, true); }
                    }
                }
            });
        }
    });

    // When a comment is submitted without a user logged in, do a two things:
    // 1. Build a redirect URL
    // 2. Open the login modal
    $('.comments-form-container-logged-out #comments-form').submit(function (event) {
        var $this = $(this),
            currentPageUrl,
            URL,
            commentContent,
            $textarea = $this.find('textarea');

        // Get base64 encoded version of comment
        commentContent = window.btoa($textarea.val());

        // Get the current listing's URL to redirect to after login or signup
        if (BaT_Theme.currentUrl.indexOf('?') > -1) {
            currentPageUrl = BaT_Theme.currentUrl + '&comment_content=' + commentContent;
        } else {
            currentPageUrl = BaT_Theme.currentUrl + '?comment_content=' + commentContent;
        }

        currentPageUrl = currentPageUrl + '#comments';

        // Build the URL with the encoded comment and current listing URL
        URL = BaT_Theme.loginUrl + '?modal=modal&comment_content=' + commentContent + '&redirect=' + encodeURIComponent(currentPageUrl);

        // Open the login modal in an iframe
        $.magnificPopup.open({
            items: {
                src: URL,
                type: 'iframe'
            },
            iframe: {
                markup: '<div><iframe class="mfp-iframe poppable" name="poppable-target" id="bat-poppable-target" frameborder="0"></iframe></div>'
            }
        });

        event.preventDefault();
    });

    // Allow subscriptions to comment by clicking a link
    $(document).on('click', '.comment-subscribe', function (event) {
        event.preventDefault();

        var $this = $(this),
            $parent = $this.parents('.subscribe-to-comments');

        if ('yes' !== $this.data('subscribing-to-comment')) {
            $this.data('subscribing-to-comment', 'yes');
            $.get(
                $this.attr('href'),
                {
                    ajax: 'yes',
                },
                function (data, status) {
                    $parent.html(data.replacement);
                },
                'json'
            );
        }
    });

    // Open new windows when share links are cliekd (Twitter, Facebook, Google+)
    $('.post-share-link[data-share-height][data-share-width]').click(function (event) {
        event.preventDefault();

        var $this = $(this);

        window.open($this.attr('href'), $this.attr('class'), 'height=' + $this.data('share-height') + ',width=' + $this.data('share-width'));
    });

    // When a user subscribes to the mailing list, submit via AJAX
    $('.constant-contact-form').ajaxForm({
        beforeSerialize: function ($form, options) {
            $form.append('<input type="hidden" name="ajax" value="ajax" />');
            $form.find('[type="submit"]').attr('disabled', 'disabled');
        },
        success: function (response, status, xhr, $form) {
            $form.empty().append('<div class="notice">' + BaT_Theme.constantContactFormSubmissionText + '</div>');

            setTimeout(function () {
                $.magnificPopup.close();
            }, 3500);
        }
    });


    $('.auction-ends-disclaimer-toggle').click(function (event) {
        event.preventDefault();

        $(this).siblings('.auctions-ends-disclaimer-text').toggle();
    });

    // Countdown timer
    $('[data-auction-ends]').each(function (index, ends) {
        var $ends = $(ends),
            layout = $ends.data('layout') ? $ends.data('layout') : '{d<}{dn} {dl}, {d>} {h<}{hn} {hl}, {h>} {m<}{mn} {ml}, {m>} {sn} {sl}',
            parts = $ends.data('auction-ends').split('-'),
            until = new Date(Date.UTC(parseInt(parts[0]), (parseInt(parts[1]) - 1), parseInt(parts[2]), parseInt(parts[3]), parseInt(parts[4]), parseInt(parts[5]), 0));

        $ends.countdown({
            alwaysExpire: true,
            onExpiry: function () {
                if ($ends.siblings('.listing-end-processing-notice').size()) {
                    $ends.siblings('.listing-end-processing-notice').show();
                    $ends.hide();
                }
            },
            format: 'ydhms',
            labels: ['years', 'months', 'weeks', 'days', 'hours', 'minutes', 'seconds'],
            labels1: ['year', 'month', 'week', 'day', 'hour', 'minute', 'second'],
            layout: layout,
            until: until
        });
    });

    $('#bat-listings-confirm-bid-form').submit(function (event) {
        $('#bat-listings-listings-place').prop('disabled', true);
    });

    // Generic modal HTML editing
    if ($('meta[name="modal"][value="yes"]').length > 0) {
        $('form').each(function (index, form) {
            var $form = $(form);

            $form.append('<input type="hidden" name="modal" value="modal" />');
        });

        $('.close-modal').click(function (event) {
            var parent = window.parent;

            if (parent && parent !== window) {
                event.preventDefault();

                if (parent.postMessage) {
                    parent.postMessage({ closeWindow: true }, '*');
                } else {
                    parent.jQuery.magnificPopup.close();
                }
            }
        });

        $('.refresh-page').click(function (event) {
            event.preventDefault();

            var parent = window.parent;

            parent.document.location.reload();
        });

        $('a.non-modal-link').click(function (event) {
            event.preventDefault();

            var url = $(this).attr('href'),
                parent = window.parent;

            parent.document.location.href = url;
        });

        setTimeout(function () {
            var height = Math.min($(window.parent).outerHeight(), $('body').outerHeight()),
                width = Math.min($(window.parent).outerWidth() - 40, 480);

            var parent = window.parent;

            if (parent && parent.postMessage) {
                parent.postMessage({ height: height, width: width }, '*');
            }
        }, 250);
    } else {
        var $poppableTarget = $('<iframe class="poppable mfp-hide" name="poppable-target" id="bat-poppable-target"></iframe>').appendTo($('body'));

        window.addEventListener('message', function (event) {
            if (event.data && event.data.height) {
                $poppableTarget.height(event.data.height);
            } else if (event.data && event.data.closeWindow) {
                if ($poppableTarget.data('nextModalUrl')) {
                    $poppableTarget.attr('src', $poppableTarget.data('nextModalUrl'));
                    $poppableTarget.data('nextModalUrl', '')
                } else {
                    jQuery.magnificPopup.close();
                }
            }
        }, false);

        window.$poppableTarget = $poppableTarget;

        $(document).on('click', '.poppable-link', function (event) {
            event.preventDefault();

            var $link = $(this),
                href = $link.attr('href'),
                hrefHasModal = href.indexOf('modal=') >= 0,
                hrefHasParams = href.indexOf('?') >= 0,
                nextModalHref = $link.data('nextModalUrl');

            $link.attr('target', 'poppable-target');

            if (!hrefHasModal) {
                $link.attr('href', href + (hrefHasParams ? '&' : '?') + 'modal=modal');
            }

            $poppableTarget.attr('src', $link.attr('href'));

            if (nextModalHref) {
                var nextHasModal = nextModalHref.indexOf('modal=') >= 0,
                    nextHasParams = nextModalHref.indexOf('?') >= 0;

                if (!nextHasModal) {
                    nextModalHref = nextModalHref + (nextHasParams ? '&' : '?') + 'modal=modal';
                }

                $poppableTarget.data('nextModalUrl', nextModalHref);
            }

            $.magnificPopup.open({
                callbacks: {
                    beforeClose: function () {
                        // jQuery.magnificPopup.close() causes the iframe to reload in background.
                        // Mark the second load with a special query string to detect when the load is invisible to the user.
                        var iframeSrc = $poppableTarget.attr('src'),
                            iframeTargetHasParams = iframeSrc.indexOf('?') >= 0,
                            modifiedIframeSrc = iframeSrc + (iframeTargetHasParams ? '&' : '?') + 'closing_modal=closing_modal';

                        $poppableTarget.attr('src', modifiedIframeSrc);
                    }
                },
                items: {
                    src: window.$poppableTarget,
                    type: 'inline'
                }
            });
        });

        $('.poppable-form').each(function (index, form) {
            var $form = $(form);

            $form.attr('target', 'poppable-target');
            $form.append('<input type="hidden" name="modal" value="modal" />');
        }).submit(function (event) {
            var $form = $(this);

            $.magnificPopup.open({
                items: {
                    src: $poppableTarget,
                    type: 'inline',
                }
            });
        });

        $('.listing-bid-form').submit(function (event) {
            setTimeout(function () {
                $('#bat-listings-listings-amount').val('');
            }, 1000);
        });

        if ('1' == $('#poppable-link-reminder-toggle').val()) {
            $('#poppable-link-reminder').trigger('click');
        }

        if ('1' == $('#poppable-link-contact-toggle').val()) {
            $('#poppable-link-contact').trigger('click');
        }
    }

    $(document).on('click', '#results-box-close-button', function (event) {
        event.preventDefault();

        $('#listing-actions-results-container').hide();

        $.post(
            BaT_Theme.ajaxUrl,
            {
                action: BaT_Theme.ajaxActionDismissListingResultsBox,
                listing: BaT_Theme.post
            },
            function (data) {
            },
            'json'
        );

    });

    // Listings page - Photo gallery (PhotoSwipe)
    var photoGallery = (function () {
        var galleryPhotosSelector = '.listing-gallery a.listing-gallery-image-container:not(#listing-gallery-item-more-template .listing-gallery-image-container)';

        var initialize = function () {
            var items = photoGallery.getGalleryItems();

            $(document).on('click', galleryPhotosSelector, function (event) {
                event.preventDefault();

                var $this = $(this),
                    $galleryPhotos = $(galleryPhotosSelector),
                    galleryIndex = $galleryPhotos.index($this),
                    photoSwipeDomElement = photoGallery.getPhotoSwipeDomElement();

                var options = {
                    index: galleryIndex,       // Start on clicked photo
                    preload: [1, 2],           // Preload 1 previous and 2 next photos
                    history: false,            // Do not modify browser history
                    clickToAdvanceImage: true, // Clicking advances image instead of zooming in
                    shareButtons: [
                        { id: 'facebook', label: BaT_Theme.shareLinkFacebookText, url: BaT_Theme.shareLinkFacebook },
                        { id: 'reddit', label: BaT_Theme.shareLinkRedditText, url: BaT_Theme.shareLinkReddit },
                        { id: 'twitter', label: BaT_Theme.shareLinkTwitterText, url: BaT_Theme.shareLinkTwitter },
                        { id: 'email', label: BaT_Theme.shareLinkEmailText, url: BaT_Theme.shareLinkEmail },
                        { id: 'download', label: 'Save', url: '{{raw_image_url}}', download: true }
                    ],
                };

                var pswp = new PhotoSwipe(photoSwipeDomElement, PhotoSwipeUI_BaT, items, options);
                pswp.init();

                pswp.listen('beforeChange', function () {
                    var $more = jQuery('.listing-gallery-item-more'),
                        moreIndex = $galleryPhotos.index($more),
                        currentIndex = pswp.getCurrentIndex();

                    // Expand additional photos on page if PhotoSwipe is viewing them
                    if ($more.size() > 0 && currentIndex >= moreIndex) {
                        $more.trigger('click');
                    }
                });
            });
        };

        var getGalleryItems = function () {
            var $galleryPhotos = $(galleryPhotosSelector),
                items = [];

            var createGalleryItem = function (dimensions, large, small, id, context, voted, votes) {
                var dimensionParts = dimensions.split('x'), galleryItem = {
                    context: context,
                    id: id,
                    msrc: small,
                    src: large,
                    voted: !!voted, // Normalize to boolean
                    votes: votes,
                    h: dimensionParts[1],
                    w: dimensionParts[0]
                };

                // Global registration for live updates
                window.galleryItems = window.galleryItems || [];
                window.galleryItems.push(galleryItem);

                return galleryItem;
            };

            // Add photos on page
            $galleryPhotos.each(function (index, element) {
                var $element = $(element),
                    $image = $element.find('img'),
                    item = createGalleryItem($element.data('dimensions'), $element.attr('href'), $image.attr('src'), $element.data('id'), $element.data('context'), $element.data('voted'), $element.data('votes'));

                items.push(item);
            });

            // Add additional (unloaded) photos on page
            var moreImageData = photoGallery.getMoreImageData();
            $.each(moreImageData, function (index, element) {
                var galleryItem = createGalleryItem(element.dimensions, element.large, element.small, element.id, element.context, element.voted, element.votes);

                items.push(galleryItem);
            });

            return items;
        }

        var getPhotoSwipeDomElement = function () {
            var $photoSwipeElement = $('.pwsp');

            if ($photoSwipeElement.length === 0) {
                var photoSwipeHtml = '<div class="pswp" tabindex="-1" role="dialog" aria-hidden="true"><div class="pswp__bg"></div><div class="pswp__scroll-wrap"><div class="pswp__container"><div class="pswp__item"></div><div class="pswp__item"></div><div class="pswp__item"></div></div><div class="pswp__ui pswp__ui--hidden"><div class="pswp__top-bar"><div class="pswp__counter"></div><button class="pswp__button pswp__button--close" title="Close (Esc)"></button><button class="pswp__button pswp__button--share" title="Share"></button><button class="pswp__button pswp__button--fs" title="Toggle fullscreen"></button><button class="pswp__button pswp__button--zoom" title="Zoom in/out"></button><span class="pswp__button pswp__button--votes"></span><div class="pswp__preloader"><div class="pswp__preloader__icn"><div class="pswp__preloader__cut"><div class="pswp__preloader__donut"></div></div></div></div></div><div class="pswp__share-modal pswp__share-modal--hidden pswp__single-tap"><div class="pswp__share-tooltip"></div></div><button class="pswp__button pswp__button--arrow--left" title="Previous (arrow left)"></button><button class="pswp__button pswp__button--arrow--right" title="Next (arrow right)"></button><div class="pswp__caption"><div class="pswp__caption__center"></div></div></div></div></div>';

                $photoSwipeElement = $(photoSwipeHtml);
                $('body').append($photoSwipeElement);
            }

            return $photoSwipeElement.get(0);
        }

        var getMoreImageData = function () {
            var $more = jQuery('.listing-gallery-item-more');

            if ($more.length === 0) {
                return [];
            } else {
                var $moreImageDataElement = $('input.listing-gallery-item-more-data'),
                    moreImageData = $.parseJSON($moreImageDataElement.val());

                return moreImageData;
            }
        }

        return {
            initialize: initialize,
            getGalleryItems: getGalleryItems,
            getPhotoSwipeDomElement: getPhotoSwipeDomElement,
            getMoreImageData: getMoreImageData
        };
    }());

    (function () {
        BaT_Theme.featuredListingsSliders = function () {
            $('.featured-listings-module:visible').each(function (index, element) {
                var $featuredListingsModule = $(element),
                    $featuredListingsWrapper = $featuredListingsModule.find('.featured-listings-wrapper'),
                    $featuredListingsOuter = $featuredListingsModule.find('.featured-listings'),
                    $featuredListings = $featuredListingsModule.find('.featured-listing'),
                    $featuredListingsPagination = $featuredListingsModule.find('.featured-listings-pagination'),
                    $featuredListingsDots = $featuredListingsModule.find('.featured-listings-dot'),
                    $featuredListingsArrows = $featuredListingsModule.find('.featured-listings-arr'),
                    automatic = null,
                    initialized = false,
                    interacted = false,
                    resize = null,
                    transition = null,
                    timeout = null;

                if (0 === $featuredListingsOuter.size() || $featuredListingsModule.data('initialized')) { return; }

                $featuredListingsModule.data('initialized', true);

                automatic = function () {
                    if (0 === $featuredListingsModule.data('automatic')) { return; }

                    var index = $featuredListingsDots.filter('.featured-listings-dot-curr').index(),
                        next = index + 1;

                    if (next >= $featuredListingsDots.size()) {
                        next = 0;
                    }

                    $featuredListingsDots.eq(next).trigger('click');
                };

                resize = function () {
                    var width = $featuredListingsWrapper.width();

                    $featuredListingsOuter.width($featuredListings.size() * width);
                    $featuredListings.width(width).addClass('featured-listing-processed');

                    // Adjust inner content to stay centered if scrollbar is present.
                    // Necessary so that next and previous listing arrows do not move horizontally if some listings have scrollbars.
                    var $modal = $('#modal-listing'),
                        $modalPopup = $modal.find('.modal-popup'),
                        isScrollBarShowing = $modal.prop('scrollHeight') > $modal.height();

                    if (isScrollBarShowing) {
                        $modalPopup.css({ 'position': 'relative', 'left': BaT_Theme.getScrollbarWidth() / 2 });
                    } else {
                        $modalPopup.css({ 'position': '', 'left': '' });
                    }
                };

                transition = function (index, immediate) {
                    var immediate = true === immediate,
                        speed = $featuredListingsOuter.data('speed') || 'medium',
                        width = $featuredListings.first().width(),
                        props = { left: (-1 * index * width) };

                    if (immediate) {
                        $featuredListingsOuter.css(props);
                    } else {
                        $featuredListingsOuter.stop().animate(props, speed);
                    }
                };

                $featuredListingsWrapper.parent().swipe({
                    swipe: function (event, direction, distance, duration, fingerCount, fingerData, currentDirection) {
                        if ('up' === direction || 'down' === direction) { event.preventDefault(); return; }

                        var addable = 'right' === direction ? -1 : 1;
                        current = $featuredListingsDots.filter('.featured-listings-dot-curr').index(),
                            index = current + addable;

                        if (index >= $featuredListingsDots.size()) {
                            index = 0;
                        } else if (index < 0) {
                            index = $featuredListingsDots.size() - 1;
                        }

                        interacted = true;
                        if (null !== timeout) {
                            clearInterval(timeout);
                        }

                        $featuredListingsDots.eq(index).trigger('click');
                    },

                    hold: function (event, target) {
                        var $listing = $(target).parents('img.featured-listing');

                        if (($(window).width() < 751 || navigator.userAgent.match(/(iPad|iPhone)/)) && $listing.size() > 0) {
                            var href = $listing.find('a.featured-listing-image-container').attr('href');

                            if (href) {
                                window.location.href = href;
                            }
                        }
                    },

                    tap: function (event, target) {
                        if (event.pointerType == 'mouse' && event.which != 1) {
                            // Ignore middle and right mouse clicks
                            return;
                        }

                        var $clickedItem = $(target);

                        if ($clickedItem.is('img') || $clickedItem.is('.listing-feature-modal-gallery-image')) {
                            $featuredListingsArrows.filter('[data-direction="next"]').click();
                        } else if ($clickedItem.is('.bat-plyr-cover')) {
                            // Prevent player controls underneath cover from firing
                            event.preventDefault();

                            $clickedItem.trigger('tap');
                        }
                    },


                    allowPageScroll: $.fn.swipe.pageScroll.VERTICAL,
                    excludedElements: '.plyr,' + $.fn.swipe.defaults.excludedElements,
                    threshold: 20
                });

                $featuredListingsPagination.show();

                $featuredListingsDots.click(function (event) {
                    event.preventDefault();

                    var $clicked = $(this);

                    if ($clicked.hasClass('featured-listings-dot-curr')) { return; }

                    var previousIndex = $featuredListingsDots.filter('.featured-listings-dot-curr').index(),
                        $previousItem = $featuredListingsOuter.children().eq(previousIndex);

                    $featuredListingsDots.removeClass('featured-listings-dot-curr').filter($clicked).addClass('featured-listings-dot-curr');

                    transition($clicked.index(), !initialized);

                    if (previousIndex >= 0) {
                        BaT_Theme.player.pauseChildren($previousItem);
                    }

                    initialized = true;

                    if ('undefined' !== typeof event.which) {
                        interacted = true;
                        if (null !== timeout) {
                            clearInterval(timeout);
                        }
                    }
                }).first().trigger('click');

                $featuredListingsArrows.click(function (event) {
                    event.preventDefault();

                    var $clicked = $(this),
                        addable = 'prev' === $clicked.data('direction') ? -1 : 1;
                    current = $featuredListingsDots.filter('.featured-listings-dot-curr').index(),
                        index = current + addable;

                    if (index >= $featuredListingsDots.size()) {
                        index = 0;
                    } else if (index < 0) {
                        index = $featuredListingsDots.size() - 1;
                    }

                    $featuredListingsDots.eq(index).trigger('click');

                    interacted = true;
                    if (null !== timeout) {
                        clearInterval(timeout);
                    }
                });

                $featuredListings.add($featuredListingsPagination).mouseenter(function () {
                    if (null !== timeout) {
                        clearInterval(timeout);

                        timeout = null;
                    }
                }).mouseleave(function () {
                    if (null === timeout && false === interacted) {
                        timeout = setInterval(automatic, 3000);
                    }
                }).first().trigger('mouseleave');

                // Remove previous window resize bindings
                $(window).off('resize.featuredListingsSliders');

                $(window).on('resize.featuredListingsSliders', function () {
                    resize();

                    transition($featuredListingsDots.filter('.featured-listings-dot-curr').index());
                });
            });

            $(window).trigger('resize');
        };

        BaT_Theme.featuredListingsOEmbeds = function () {
            var $module = $('.featured-listings-module:visible'),
                $oembeds = $module.find('.bat-oembed-plyr');

            $oembeds.each(function (index, element) {
                var $oembed = $(element);

                // Hide oembed while loading
                $oembed.css('visibility', 'hidden');

                BaT_Theme.player.initialize($oembed);
                var player = BaT_Theme.player.get($oembed);

                player.on('ready', function (event) {
                    $oembed.css('visibility', 'visible').hide().fadeIn('slow');
                });

                player.on('playing', function (event) {
                    // Sometimes the player cover will incorrectly show up in front of videos that are playing.
                    // Make sure it's disabled whenever the video is playing.
                    var $playerElement = $(player.elements.container),
                        $videoCover = $playerElement.siblings('.bat-plyr-cover');

                    if ($videoCover.is(':visible')) {
                        $videoCover.hide();
                    }
                });

                // Prevent double clicking on video to enter fullscreen. Can cause accidental fullscreen when clicking from video cover.
                player.eventListeners.forEach(function (eventListener) {
                    if (eventListener.type === 'dblclick') {
                        element.removeEventListener(eventListener.type, eventListener.callback, eventListener.options);
                    };
                });
            });
        }

        var showPlayerCover = function (event) {
            var player = event.detail.plyr,
                $playerElement = $(player.elements.container),
                $videoCover = $playerElement.siblings('.bat-plyr-cover');

            $videoCover.show();
        }

        $(document).on('tap', '.listing-feature-modal-gallery .bat-plyr-cover', function (event) {
            var $videoCover = $(this),
                $playerElement = $videoCover.closest('.listing-feature-modal-gallery-video').find('.plyr'),
                player = BaT_Theme.player.get($playerElement);

            $videoCover.hide();

            player.play();

            // Remove previous bat-pause handlers
            $playerElement.off('bat-pause', showPlayerCover);

            $playerElement.on('bat-pause', showPlayerCover);
        });

        BaT_Theme.featuredListingsSliders();
        BaT_Theme.featuredListingsCountdown();
    }());

    (function () { // initialize slider for popular listings
        BaT_Theme.popularListingsSliders = function () {
            var popularSwiper = new Swiper('.popular-listings-module .swiper-container', {
                pagination: {
                    el: '.popular-listings__pagination',
                    type: 'bullets',
                    bulletClass: 'featured-listings-dot',
                    bulletActiveClass: 'featured-listings-dot-curr',
                    clickable: true,
                },
                navigation: {
                    nextEl: '.featured-listings-arr-next',
                    prevEl: '.featured-listings-arr-prev',
                },
                loop: true,
                a11y: true,
                autoHeight: true,
                slidesPerView: 4,
                slidesPerGroup: 4,
                spaceBetween: 20,
                // Responsive breakpoints
                breakpoints: {
                    // when window width is <= 480px
                    480: {
                        slidesPerView: 2,
                        slidesPerGroup: 2,
                        spaceBetween: 20
                    },
                }
            });
        };

        BaT_Theme.popularListingsSliders();
    }());

    (function () { // initialize slider for exceptional listings
        BaT_Theme.exceptionalListingsSliders = function () {
            var exceptionalSwiper = new Swiper('.exceptional-listings-module .swiper-container', {
                pagination: {
                    el: '.exceptional-listings__pagination',
                    type: 'bullets',
                    bulletClass: 'featured-listings-dot',
                    bulletActiveClass: 'featured-listings-dot-curr',
                    clickable: true,
                },
                navigation: {
                    nextEl: '.featured-listings-arr-next',
                    prevEl: '.featured-listings-arr-prev',
                },
                loop: true,
                a11y: true,
                autoHeight: true,
                slidesPerView: 4,
                slidesPerGroup: 4,
                spaceBetween: 20,
                // Responsive breakpoints
                breakpoints: {
                    // when window width is <= 480px
                    480: {
                        slidesPerView: 2,
                        slidesPerGroup: 2,
                        spaceBetween: 20
                    },
                }
            });
        };

        BaT_Theme.exceptionalListingsSliders();
    }());

    photoGallery.initialize();

    $('.listing-gallery-item-more').click(function (event) {
        event.preventDefault();
        event.stopPropagation();

        var $this = $(this),
            $img = $this.find('img.listing-gallery-image-more'),
            $cover = $this.find('.listing-gallery-item-more-cover'),
            $text = $this.find('.listing-gallery-item-more-text'),
            $clear = $('.listing-gallery-clear'),
            morePhotoData = photoGallery.getMoreImageData(),
            template = $('#listing-gallery-item-more-template').html();

        $cover.remove();
        $text.remove();
        $img.removeClass('listing-gallery-image-more');

        $this.removeClass('listing-gallery-item-more');
        $this.off(event);

        for (var i = 0; i < morePhotoData.length; i++) {
            var $item = $(template
                .replace(/#context/g, morePhotoData[i].context)
                .replace(/#dimensions/g, morePhotoData[i].dimensions)
                .replace(/#id/g, morePhotoData[i].id)
                .replace(/#large/g, morePhotoData[i].large)
                .replace(/#small/g, morePhotoData[i].small)
                .replace(/#voted/g, morePhotoData[i].voted)
                .replace(/#votes/g, morePhotoData[i].votes)
            );

            $item.insertBefore($clear);
        }
    });

    (function () {
        $('.comments-time-check').each(function (index, check) {
            var $check = $(check),
                timeB = parseInt((new Date()).getTime() / 1000),
                timeR = $check.data('timestamp'),
                timeD = Math.abs(timeR - timeB);

            if (timeD > BaT_Theme.timeThreshold) {
                $check.show();

                $.post(
                    BaT_Theme.ajaxUrl,
                    {
                        action: 'bat_check_time',
                        post: BaT_Theme.post,
                        timeB: timeB,
                        timeR: timeR
                    },
                    function () { },
                    'json'
                );
            }
        });
    }());

    // var $commentsPagination = $('.search-results .pagination').addClass('hide-buttons').data('fetching', false);
    var commentsPaginationSelector = '.member-profile .pagination-comments';
    var commentsListingSelector = '.member-profile-section-comments';
    var commentsItemSelector = '.member-profile-item';
    addInlinePagination(commentsPaginationSelector, commentsListingSelector, commentsItemSelector, 'Loading more comments&hellip;');

    var bidsPaginationSelector = '.member-profile .pagination-auctions';
    var bidsListingSelector = '.member-profile-section-history';
    var bidsItemSelector = '.member-profile-item';
    addInlinePagination(bidsPaginationSelector, bidsListingSelector, bidsItemSelector, 'Loading more bids&hellip;');

    var previousAuctionsPaginationSelector = '.listings-archive .pagination-auctions .pagination';
    var previousAuctionsListingSelector = '.previous-listings';
    var previousAuctionsItemSelector = '.previous-listing';
    addInlinePagination(previousAuctionsPaginationSelector, previousAuctionsListingSelector, previousAuctionsItemSelector, 'Loading more auctions&hellip;');

    var searchAuctionsPaginationSelector = '.search-results-auctions .pagination';
    var searchAuctionsListingSelector = '.search-results-auctions .previous-listings-search';
    var searchAuctionsItemSelector = '.previous-listing';
    addInlinePagination(searchAuctionsPaginationSelector, searchAuctionsListingSelector, searchAuctionsItemSelector, 'Loading more results&hellip;');

    var searchStoriesPaginationSelector = '.search-results-stories .pagination';
    var searchStoriesListingSelector = '.search-results-stories .previous-listings-search';
    var searchStoriesItemSelector = '.previous-listing';
    addInlinePagination(searchStoriesPaginationSelector, searchStoriesListingSelector, searchStoriesItemSelector, 'Loading more results&hellip;');

    $('.bat-listings-users-form-register-payment #bat-listings-users-referral_source').change(function () {
        var selectedOption = $(this).val();
        $('#bat-listings-users-referral_descriptor').val('').parents('.control-group').toggle('referred' === selectedOption || 'other' === selectedOption).find('label').hide().filter(function (index, label) {
            return $(label).data('referral_descriptor_label') === selectedOption;
        }).show();
    }).trigger('change');

    $('.sidebar').imagesLoaded(batFixAdrotateAd);
});

function addInlinePagination(paginationSelector, listingSelector, itemSelector, message) {
    var $ = jQuery;

    var $memberCommentsPagination = $(paginationSelector);
    $memberCommentsPagination.on('click', '[data-older]', function (e) {
        e.preventDefault();

        $memberCommentsPagination.addClass('hide-buttons');

        var $listings = $(listingSelector),
            $link = $(this).parent(),
            $text = $('<div style="text-align:center;"><em>' + message + '</em></div>');

        if ($link.size()) {
            $memberCommentsPagination.append($text).find('a').remove();

            $.get(
                $link.attr('href'),
                {},
                function (data, status) {
                    var $page = $(data),
                        $results = $page.find(listingSelector).find(itemSelector),
                        $pagination = $page.find(paginationSelector);

                    // jQuery doesn't attach to new pagination links in the document instantly
                    // Prevent pagination clicks that might cause a /page/x/ to load
                    $pagination.find('a').on('click', function (e) { e.preventDefault(); });

                    $listings.append($results);
                    $memberCommentsPagination.append($pagination.find('a')).removeClass('hide-buttons');
                    $text.remove();
                }
            );
        }
    });
}

function batFixAdrotateAd() {
    var fixedAd = jQuery('#bat_adrotate_fixed');
    var fixedAdTopMargin = 40;

    if (fixedAd.length) {
        var fixedAdTop = fixedAd.offset().top - fixedAdTopMargin;

        var positionAd = function () {
            var currentScroll = jQuery(window).scrollTop();

            var stickyHeader = jQuery('.listing-sticky-fixed');

            var adTopMargin = fixedAdTopMargin;
            var adTop = fixedAdTop;
            if (stickyHeader.length) {
                adTopMargin += stickyHeader.height();
                adTop -= stickyHeader.height();
            }

            if (currentScroll >= adTop) {
                fixedAd.css({
                    position: 'fixed',
                    top: adTopMargin + 'px',
                });
            } else {
                fixedAd.css({
                    position: 'static'
                });
            }
        };

        jQuery(window).scroll(positionAd);

        jQuery(window).resize(function () {
            fixedAd.css({
                position: 'static'
            });

            fixedAdTop = fixedAd.offset().top - fixedAdTopMargin;
            positionAd();
        });

        positionAd();
    }
}

BaT_Theme.isScrolledIntoView = function (elem) {
    var $elem = jQuery(elem);
    var $window = jQuery(window);

    if ($elem.size()) {
        var docViewTop = $window.scrollTop();
        var docViewBottom = docViewTop + $window.height();

        var elemTop = $elem.offset().top;
        var elemBottom = elemTop + $elem.height();

        return ((elemBottom <= docViewBottom) && (elemTop >= docViewTop));
    } else {
        return false;
    }
}

BaT_Theme.getUserLocation = function () {
    var userLocation = false;
    if (localStorage.getItem(BaT_Theme.userLocationStorageKey)) {
        userLocation = JSON.parse(localStorage.getItem(BaT_Theme.userLocationStorageKey))
    } else if (BaT_Theme.closestToZipUserProfileLocation) {
        userLocation = BaT_Theme.closestToZipUserProfileLocation;
    }

    return userLocation;
}

BaT_Theme.setUserLocation = function (userLocation) {
    localStorage.setItem(BaT_Theme.userLocationStorageKey, JSON.stringify(userLocation));
}

BaT_Theme.promptUserLocation = function (success, failure) {
    jQuery('#bat-auctions-set-zip-code').trigger('click');

    var afterClose = function(event) {
        if (typeof failure === 'function') {
            failure();
        }
    };

    jQuery(document).one('setZipCodeClosed', function (event) {
        if (typeof success === 'function') {
            success();
        }

        jQuery(document).off('mfpAfterClose', afterClose);
    }).one('mfpAfterClose', afterClose);
}

BaT_Theme.guid = function () {
    function s4() { return Math.floor((1 + Math.random()) * 0x10000).toString(16).substring(1); }

    return s4() + s4() + '-' + s4() + '-' + s4() + '-' + s4() + '-' + s4() + s4() + s4();
}

BaT_Theme.player = (function () {
    var $ = jQuery;
    var instances = {};

    // Initializes Plyr instances within the passed item
    var initialize = function ($item, options) {
        if (options === undefined) {
            options = {};
        }

        var $oembeds = $item.find('.bat-oembed-plyr.plyr__video-embed').addBack('.bat-oembed-plyr.plyr__video-embed');

        $oembeds.each(function (index, element) {
            var $element = $(element),
                guid = BaT_Theme.guid();

            $element.attr('id', guid);

            // YouTube oembed feature conflicts with Plyr controls on iOS. Remove the oembed query parameter from embedded iframes.
            removeOembedQueryParameter($element);

            instances[guid] = new Plyr(element, options);
            addBatPauseEvent(instances[guid]);
        });
    };

    // Return the first Plyr instance within the passed item
    var get = function ($item) {
        var plyrId = $item.find('.plyr').addBack('.plyr').attr('id');

        if (instances[plyrId]) {
            return instances[plyrId];
        } else {
            return false;
        }
    };

    // Pause any videos within the passed item
    var pauseChildren = function ($item) {
        var $players = $item.find('.plyr--playing');

        $players.each(function () {
            var $element = $(this);

            var player = BaT_Theme.player.get($element);
            player.pause();
        });
    };

    // The 'pause' event from the player is not a good indicatior if the user actually paused the video. Use 'bat-pause' event instead.
    var addBatPauseEvent = function (player) {
        // When a user is seeking, the 'pause' event fires followed by one or more 'seeking' events. In order to determine if the video was actually
        // paused versus when a user was seeking, delay and check if the user was seeking.
        player.on('seeking', function (event) {
            var player = event.detail.plyr,
                $playerElement = $(player.elements.container);

            $playerElement.data('bat-plyr-is-seeking', true);
        });

        // When a user finished seeking, save when the action completed to determine if the seeking finished during the setTimeout delay below
        player.on('seeked', function (event) {
            var player = event.detail.plyr,
                $playerElement = $(player.elements.container);

            $playerElement.data('bat-plyr-is-seeking', false);
            $playerElement.data('bat-plyr-finished-seeking-time-ms', Date.now());
        });

        // When a video resumes playing, save when the action completed to determine if video was resumed during the setTimeout delay below
        player.on('playing', function (event) {
            var player = event.detail.plyr,
                $playerElement = $(player.elements.container);

            $playerElement.data('bat-plyr-started-playing-time-ms', Date.now());
        });

        player.on('pause', function (event) {
            var player = event.detail.plyr,
                $playerElement = $(player.elements.container),
                seekCheckDelayMs = 100;

            setTimeout(function () {
                var isSeeking = $playerElement.data('bat-plyr-is-seeking'),
                    currentTimeMs = Date.now(),
                    isFinishedSeekingDuringTimeout = (currentTimeMs - $playerElement.data('bat-plyr-finished-seeking-time-ms')) <= seekCheckDelayMs * 1.5,
                    isStartedPlayingDuringTimeout = (currentTimeMs - $playerElement.data('bat-plyr-started-playing-time-ms')) <= seekCheckDelayMs * 1.5;

                if (isSeeking || isFinishedSeekingDuringTimeout || isStartedPlayingDuringTimeout) {
                    // Ignore this 'pause', the user is seeking or resumed watching the video
                } else {
                    var event = new Event('bat-pause');
                    event.detail = event.detail || {};
                    event.detail.plyr = player;

                    $playerElement.get(0).dispatchEvent(event);
                }
            }, seekCheckDelayMs);
        });
    };

    var removeOembedQueryParameter = function ($playerElement) {
        $playerElement.find('iframe').each(function (index, iframe) {
            var $iframe = $(iframe),
                iframeSrc = $iframe.attr('src');

            if (iframeSrc.indexOf('feature=oembed') > -1) {
                iframeSrc = iframeSrc.replace('feature=oembed', '');
                $iframe.attr('src', iframeSrc);
            }
        });
    };

    return {
        initialize: initialize,
        get: get,
        pauseChildren: pauseChildren,
    };
})();

BaT_Theme.getScrollbarWidth = function () {
    var $outer = jQuery('<div>').css({ visibility: 'hidden', width: 100, overflow: 'scroll' }).appendTo('body'),
        widthWithScroll = jQuery('<div>').css({ width: '100%' }).appendTo($outer).outerWidth();
    $outer.remove();

    return 100 - widthWithScroll;
}
