if('undefined' !== Stripe) {
	Stripe.setPublishableKey(BaT_Listings_Stripe.publishableKey);
}

if('undefined' !== stripe) {
	var stripe = Stripe(BaT_Listings_Stripe.publishableKey);
	BaT_Listings_Stripe.stripe = stripe;
}

jQuery(document).ready(function($) {
	$('.bat-listings-form-stripe').submit(function(event) {
		var _form = this,
			$form = $(_form),
			$controlGroup = $form.find('#control-group-credit-card'),
			$creditCard = $form.find('#bat-listings-credit-card-number,[data-stripe="number"]'),
			$stripeToken = $form.find('#bat-listings-stripe-token,[data-token="1"]');

		$form.find('input[type="submit"]').prop('disabled', true);

		if('yes' === $form.data('stripe-required') || '' !== $creditCard.val()) {
			event.preventDefault();

			$controlGroup.find('label.control-group-error').remove();
			Stripe.card.createToken(_form, function(status, response) {
				if(response.error) {
					$controlGroup
						.addClass('control-group-contains-text control-group-errors')
						.append('<label for="' + $creditCard.first().attr('id') + '" class="control-group-error">' + BaT_Listings_Stripe.errorMessageCardGeneric + '</label>');

					$controlGroup.find('input').focus();

					$form.find('input[type="submit"]').prop('disabled', false);
				} else {
					$stripeToken.val(response.id);
					_form.submit();
				}
			});
		}
	});

	$('.bat-listings-form-stripe-optional').submit(function(event) {
		var _form = this,
			$form = $(_form),
			$submitButton = $form.find('input[type="submit"]');

		if($form.hasClass('bat-listings-form-stripe-v3-setup-card-with-payment') || $form.hasClass('bat-listings-form-stripe-v3-setup-card-without-payment') || $form.hasClass('bat-listings-form-stripe-v3-existing-card-payment')) {
			// If this Stripe-optional form has a Stripe handler attached, use that instead
			return;
		}

		event.preventDefault();

		if($form.hasClass('bat-listings-form-stripe-validate') && !$form.valid()) {
			return;
		}

		$submitButton.prop('disabled', true);

		// Stripe-optional forms will need to be submitted via AJAX.
		$form.ajaxSubmit({
			dataType: 'json',
			success: function(response, statusText, xhr, $form) {
				// Validation errors
				if(response.success === false) {
					var errors = response.data.errors;
					BaT_Listings_Stripe.handleValidationErrors(errors, $form);
				// Successful redirection
				} else if(response.success === true && response.data.redirect) {
					window.location.replace(response.data.redirect);
				}
			},
		});
	});

	$('.bat-listings-form-stripe-v3-setup-card-with-payment').submit(function(event) {
		var _form = this,
			$form = $(_form),
			$errorNotice = $('.notice.error'),
			$controlGroups = $form.find('.control-group'),
			$creditCard = $form.find('#bat-listings-credit-card-number,[data-stripe="number"]'),
			$stripeToken = $form.find('#bat-listings-stripe-token,[data-token="1"]'),
			$submitButton = $form.find('input[type="submit"]');

		event.preventDefault();

		if('yes' !== $form.data('stripe-required') && '' === $creditCard.val()) {
			return;
		} else if($form.hasClass('bat-listings-form-stripe-validate') && !$form.valid()) {
			return;
		}

		$submitButton.prop('disabled', true);

		$controlGroups.removeClass('control-group-contains-text control-group-errors');
		$controlGroups.find('label.control-group-error').remove();
		$errorNotice.hide();

		Stripe.card.createToken(_form, function(status, response) {
			if(response.error) {
				BaT_Listings_Stripe.handleStripeError($form, response.error);
			} else {
				$stripeToken.val(response.id);

				$form.ajaxSubmit({
					dataType: 'json',
					success: function(response, statusText, xhr, $form) {
						// Validation errors
						if(response.success === false) {
							var errors = response.data.errors;
							BaT_Listings_Stripe.handleValidationErrors(errors, $form);

							$form.find('input[name="payment_intent_id"]').remove();
						// SCA validation
						} else if(response.requires_action === true) {
							BaT_Listings_Stripe.handleStripeAction(response, $form);
						// Redirection
						} else if(response.success === true && response.data.redirect) {
							window.location.replace(response.data.redirect);
						}
					},
				});
			}
		});
	});

	$(document).on('submit', '.bat-listings-form-stripe-v3-setup-card-without-payment', function(event) {
		var _form = this,
			$form = $(_form),
			$errorNotice = $('.notice.error'),
			$controlGroups = $form.find('.control-group'),
			$creditCard = $form.find('#bat-listings-credit-card-number,[data-stripe="number"]'),
			$setupIntent = $form.find('#bat-listings-stripe-setup-intent'),
			$paymentMethodId = $form.find('#bat-listings-stripe-successful-payment-method-id'),
			$submitButton = $form.find('input[type="submit"]');

		event.preventDefault();

		if($form.hasClass('bat-listings-form-stripe-validate') && !$form.valid()) {
			return;
		}

		$submitButton.prop('disabled', true);

		$controlGroups.removeClass('control-group-contains-text control-group-errors');
		$controlGroups.find('label.control-group-error').remove();
		$errorNotice.hide();

		var handleBatResponse = function(response) {
			// Validation errors
			if(response.success === false) {
				var errors = response.data.errors;
				BaT_Listings_Stripe.handleValidationErrors(errors, $form);
			// Redirection
			} else if(response.success === true && response.data.redirect) {
				window.location.replace(response.data.redirect);
			}
		};

		if('yes' !== $form.data('stripe-required') && '' === $creditCard.val()) {
			// Submission without Stripe
			return $form.ajaxSubmit({
				dataType: 'json',
				success: handleBatResponse
			});
		}

		// If payment method has already been created on this page, bypass Stripe setup
		if($paymentMethodId.val()) {
			return $form.ajaxSubmit({
				dataType: 'json',
				success: handleBatResponse
			});
		}

		// Create a token for the card on the page
		Stripe.card.createToken(_form, function(status, tokenResponse) {
			if(tokenResponse.error) {
				BaT_Listings_Stripe.handleStripeError($form, tokenResponse.error);
			} else {
				var stripeToken = tokenResponse.id,
					setupIntentSecret = $setupIntent.val();

				// Handle additional SCA setup for card if necessary
				stripe.handleCardSetup(setupIntentSecret, {
					'payment_method_data': {
						'card[token]': stripeToken,
					}
				}).then(function(setupResponse) {
					if(setupResponse.error) {
						BaT_Listings_Stripe.handleStripeError($form, setupResponse.error);
					} else if (setupResponse.setupIntent) {
						var paymentMethodId = setupResponse.setupIntent.payment_method;
						$paymentMethodId.val(paymentMethodId);

						// Once the payment method is approved, the original setupIntentSecret can not be reused.
						// Disable payment-related fields to indicate the payment portion of the form is complete.
						var $creditCardControlGroup = $creditCard.closest('.control-group'),
							cardNumberStars = Array($creditCard.val().length - 3).join('*'),
							cardNumberLastFour = $creditCard.val().substr($creditCard.val().length - 4);

						$form.find('.control-group-payment').not($creditCardControlGroup).slideUp();
						$creditCard.val(cardNumberStars + cardNumberLastFour);
						$creditCard.attr('disabled', true);

						$form.ajaxSubmit({
							dataType: 'json',
							success: handleBatResponse
						});
					}
				});
			}
		});
	});

	$('.bat-listings-form-stripe-v3-existing-card-payment').submit(function(event) {
		var $form = $(this),
			$errorNotice = $('.notice.error'),
			$controlGroups = $form.find('.control-group'),
			$submitButton = $form.find('input[type="submit"]');

		event.preventDefault();

		if($form.hasClass('bat-listings-form-stripe-validate') && !$form.valid()) {
			return;
		}

		$submitButton.prop('disabled', true);

		$controlGroups.removeClass('control-group-contains-text control-group-errors');
		$controlGroups.find('label.control-group-error').remove();
		$errorNotice.hide();

		$form.ajaxSubmit({
			dataType: 'json',
			success: function(response, statusText, xhr, $form) {
				// Validation errors
				if(response.success === false) {
					var errors = response.data.errors;
					BaT_Listings_Stripe.handleValidationErrors(errors, $form);

					$form.find('input[name="payment_intent_id"]').remove();
				// SCA validation
				} else if(response.requires_action === true) {
					BaT_Listings_Stripe.handleStripeAction(response, $form);
				// Redirection
				} else if(response.success === true && response.data.redirect) {
					window.location.replace(response.data.redirect);
				}
			},
		});
	});

	// Setup defaults for frontend-validated forms with jQuery Validation plugin
	$('.bat-listings-form-stripe-validate').each(function(index, form) {
		var $form = $(form),
			$errorNotice = $('.notice.error');

		$form.validate({
			errorClass: 'control-group-error',
			ignore: ':hidden:not(.bat-listings-form-validate-hidden)',
			onfocusout: false, /* disable validation checking on input blur */
			onkeyup: false, /* disable validation while typing */
			errorPlacement: function($error, $element) {
				var $controlGroup = $element.closest('.control-group');
				$error.appendTo($controlGroup);
			},
			invalidHandler: function(event, validator) {
				if(validator.numberOfInvalids() > 0) {
					$errorNotice.show();
				} else {
					$errorNotice.hide();
				}
			},
		});
	});

	BaT_Listings_Stripe.handleStripeAction = function(response, $form) {
		stripe.handleCardAction(
			response.payment_intent_client_secret
		).then(function(result) {
			if (result.error) {
				$form.find('input[name="payment_intent_id"]').remove();

				BaT_Listings_Stripe.handleStripeError($form, result.error);
			} else {
				// Resubmit the form with the approved payment intent
				$('<input type="hidden" name="payment_intent_id" />')
					.val(result.paymentIntent.id)
					.appendTo($form);

				$form.submit();
			}
		});
	};

	BaT_Listings_Stripe.handleValidationErrors = function(errors, $form) {
		var $errorNotice = $('.notice.error'),
			$submitButton = $form.find('input[type="submit"]'),
			$errorTemplate = $('<label class="control-group-error"></label>');

		$errorNotice.show();

		$.each(errors, function(inputKey, inputErrors) {
			var $input = $form.find('input, select').filter('[name="' + inputKey + '"], [name="bat-listings-users[' + inputKey + ']"], [id="' + inputKey + '"]').first(),
				$controlGroup = $input.closest('.control-group');

			if($input.length === 0) {
				// Default to general error location if specified input is not available
				$input = $form.find('input.general-error');
			}

			$controlGroup.addClass('control-group-contains-text control-group-errors');

			$.each(inputErrors, function(index, inputError) {
				var $error = $errorTemplate.clone().text(inputError).attr('for', inputKey);
				$controlGroup.append($error);
			});
		});

		// Reset reCAPTCHA if present, as previous verification is now invalid
		if($('.g-recaptcha').length > 0 && typeof(grecaptcha) !== 'undefined') {
			grecaptcha.reset();
		}

		$submitButton.prop('disabled', false);
	}

	BaT_Listings_Stripe.handleStripeError = function($form, error) {
		var $creditCard = $form.find('#bat-listings-credit-card-number,[data-stripe="number"]');

		if($creditCard.length > 0) {
			// Show error under credit card input if present
			var creditCardId = $creditCard.attr('id'),
				errors = {};
				errorMessage = BaT_Listings_Stripe.errorMessageCardGeneric;

			if ('card_declined' === error.code) {
				errorMessage = BaT_Listings_Stripe.errorMessageCardDeclined;
			} else if ('setup_intent_authentication_failure' === error.code) {
				errorMessage = BaT_Listings_Stripe.errorMessageCard3dsFailure;
			}

			errors[creditCardId] = [errorMessage]
			BaT_Listings_Stripe.handleValidationErrors(errors, $form);

			$creditCard.focus();
		} else {
			// Show error on order input for forms without a credit card input
			BaT_Listings_Stripe.handleValidationErrors({
				'general-error': [BaT_Listings_Stripe.errorMessagePayment],
			}, $form);
		}
	};
});
