(function() {
	try {
		// Connect to Pusher using the specified application key, ensuring that connections are encrypted
		BAT_WEBSERVICES_PUSHER.client = new Pusher(BAT_WEBSERVICES_PUSHER.key, BAT_WEBSERVICES_PUSHER.settings);

		// Utility function for binding to one or more channels for a specific event
		BAT_WEBSERVICES_PUSHER.bind = function(channelName, eventName, callback) {
			if(true === channelName) {
				// Bind globally which means that the event will be triggered for all subscribed channels
				BAT_WEBSERVICES_PUSHER.client.bind(eventName, callback);

			} else if('undefined' !== typeof BAT_WEBSERVICES_PUSHER.client.channel(channelName)) {
				// Bind to a specific subscribed channel
				var channel = BAT_WEBSERVICES_PUSHER.client.channel(channelName);

				channel.bind(eventName, callback);
			}
		};

		// Utility function for subscribing to a channel by name, ensuring we don't double subscribe
		BAT_WEBSERVICES_PUSHER.subscribe = function(channelName) {
			if('undefined' === typeof BAT_WEBSERVICES_PUSHER.client.channel(channelName)) {
				return BAT_WEBSERVICES_PUSHER.client.subscribe(channelName);
			}
		};
	} catch(exception) {
		BAT_WEBSERVICES_PUSHER.bind = function(channelName, eventName, callback) {
			console.log("CANNOT BIND - NO CLIENT CONNECTION");
		};

		BAT_WEBSERVICES_PUSHER.subscribe = function(channelName) {
			console.log("CANNOT SUBSCRIBE - NO CLIENT CONNECTION");
		};
	}
}());

jQuery(document).ready(function($) {
	// Bind to state change events
	BAT_WEBSERVICES_PUSHER.connected = true;

	if('object' === typeof BAT_WEBSERVICES_PUSHER.client) {
		BAT_WEBSERVICES_PUSHER.client.connection.bind('state_change', function(states) {
			if('unavailable' != states.previous && 'connected' != states.current) {
				jQuery('.pusher-connection-status').hide().filter('[data-status="' + states.current + '"]').show();
			}

			var offStates = ['unavailable', 'connecting'];
			if(offStates.includes(states.current)) {
				BAT_WEBSERVICES_PUSHER.connected = false;
			}

			if('connecting' === states.previous && 'connected' == states.current) {
				if(false === BAT_WEBSERVICES_PUSHER.connected) {
					BAT_WEBSERVICES_PUSHER.connected = true;

					connectionRestored();
				}
			}
		});
	}

	var connectionLost = function() {
		jQuery('.pusher-connection-status').hide().filter('[data-status="connecting"]').show();
	};

	var connectionRestored = function() {
		jQuery('.pusher-connection-status').hide();

		return false;

		jQuery('.pusher-connection-status').hide().filter('[data-status="fetching"]').show();

		var CLVM        = 'undefined' !== typeof BAT_VMS && 'undefined' !== typeof BAT_VMS.CLVM ? BAT_VMS.CLVM : false,
			comments    = false !== CLVM ? CLVM.comments() : [],
			lastComment = 0 === comments.length ? 0 : comments[0].id();

		if(CLVM) {
			jQuery.post(
				BaT_Theme.ajaxUrl,
				{
					action:       BAT_WEBSERVICES_PUSHER.ajaxActionListingMissedComments,
					listing_id:   BaT_Theme.post,
					last_comment: lastComment
				},
				function(data, status) {
					if(CLVM) {
						BAT_VMS.CLVM.slideInActive = false;

						jQuery.each(data.comments, function(index, comment) {
							if('undefined' !== typeof comment.end_timestamp) {
								updateCountdowns(comment.post, comment.end_timestamp);
							}

							CLVM.addComment(comment, false);
						});

						BAT_VMS.CLVM.slideInActive = true;

						jQuery('.listing-available-info span:first-child .data-value').html(data.high_bid);
						jQuery('.current-bid-value').html(data.currency + data.high_bid);
						jQuery('[data-listing-current-bidding="' + BaT_Theme.post + '"]').data('amount', data.high_bid);
						jQuery('[data-listing-current-bidding="' + BaT_Theme.post + '"]').data('high-bidder', data.high_bidder);
						jQuery('[data-listing-current-bidding="' + BaT_Theme.post + '"]').data('minimum', data.minimum_bid_numeric);
						jQuery('.number-bids-value').html(data.bid_count);
						jQuery('.comments-title').html(data.comments_title);
						jQuery('#bat-listings-listings-amount').attr('placeholder', data.minimum_text);
					}

					jQuery('.pusher-connection-status').hide();
				},
				'json'
			);
		}
	};

	window.addEventListener('offline', function() { if(true === BAT_WEBSERVICES_PUSHER.connected) { BAT_WEBSERVICES_PUSHER.connected = false; connectionLost(); } });
	window.addEventListener('online', function() { if(false === BAT_WEBSERVICES_PUSHER.connected) { BAT_WEBSERVICES_PUSHER.connected = true; connectionRestored(); } });

	// For each element on the page that has a pusher data attribute, subscribe
	// to the channel indicated by that attribute
	$('[data-pusher]').each(function(index, element) {
		var channel = $.trim($(element).data('pusher'));

		BAT_WEBSERVICES_PUSHER.subscribe(channel);
	});

	var updateCountdowns = function(post_id, end_timestamp) {
		var theme = 'undefined' !== typeof BaT_Theme ? BaT_Theme : false;

		if(theme) {
			var ends = new Date(parseInt(end_timestamp) * 1000),
				now  = new Date();

			if(ends > now) {
				var datetimem = moment(ends),
					formatted = datetimem.format('dddd, MMMM D [at] h:mma');

				$('[data-ends-id="' + post_id + '"]').text(formatted);

				var $countdowns = $('[data-countdown="' + post_id + '"],[data-auction-ends-id="' + post_id + '"],[data-recent_bid_auction_id="' + post_id + '"],[data-featured_listing="' + post_id + '"]');

				$countdowns.each(function(index, countdown) {
					var $countdown = $(countdown),
						_ends      = $countdown.countdown('option', 'until');

					if(ends != _ends) {
						$countdown.countdown('option', 'until', ends).show();
						$countdown.data('processed_final', false).data('processed_second', false).data('processed', false)
						$countdown.siblings('.listing-end-processing-notice').hide();
					}
				});

				var $listingActionsReserve = $('.listing-actions-reserve');

				if($listingActionsReserve.size()) {
					$.post(
						theme.ajaxUrl,
						{
							action:  theme.ajaxActionListingReserveBox,
							listing: theme.post
						},
						function(data, status) {
							if(data.reserve_met) {
								$('#listing-action-reserve-warn').remove();
							}

							$('.listing-actions-reserve').replaceWith(data.markup);
						}
					);
				}
			}
		}
	};

	// Comment added or updated
	BAT_WEBSERVICES_PUSHER.bind(true, 'comment-added', function(data) {
		var CLVM = 'undefined' !== typeof BAT_VMS && 'undefined' !== typeof BAT_VMS.CLVM ? BAT_VMS.CLVM : false;

		if(CLVM && parseInt(data.comment.post) === parseInt(CLVM.id())) {
			// Auction has ended
			if('bat-bid-reserve' === data.comment.type && data.comment.biddable_markup && data.comment.biddable_markup.intro && data.comment.biddable_markup.box) {
				var $listingAvailable    = $('.listing-available:first'),
					$listingBid          = $('#listing-bid-container'),
					$listingAvailableNew = $(data.comment.biddable_markup.intro).find('.listing-available'),
					$listingBidNew       = $(data.comment.biddable_markup.box),
					$biddingRow          = $('#listing-bid-row'),
					$contactSeller       = $('#listing-essentials-item-contact'),
					$sellerContactWarn   = $('#listing-action-contact-warn'),
					$sellerReserveWarn   = $('#listing-action-reserve-warn');

				// Remove the countdown, the label that points at it, and the separator - this is markup dependent
				$listingAvailable.find('[data-countdown]').countdown('destroy').parent().hide().prev().hide().prev().hide();
				$listingBid.find('[data-auction-ends-id]').countdown('destroy').parent().children().hide();

				$('.listing-available-progress-bar').remove();
				$('.listing-actions-reserve').remove();
				$sellerContactWarn.remove();
				$sellerReserveWarn.remove();
				$contactSeller.remove();

				var callback = function() {
					$listingAvailable.replaceWith($listingAvailableNew);
					$listingBid.replaceWith($listingBidNew);
				};

				$biddingRow.remove();

				if(data.comment.biddable_markup.sold_notice) {
					var $notice = $(data.comment.biddable_markup.sold_notice);

					$listingBid.find('table tbody').append($notice);

					setTimeout(function() {
						$notice.find('.listing-stats-label-centered').addClass('listing-stats-label-centered-show');

						setTimeout(function() {
							$notice.find('.listing-stats-label-centered').removeClass('listing-stats-label-centered-show');

							setTimeout(callback, 2000);
						}, 5000);

					}, 1);
				} else {
					callback();
				}

				var $listingActionsResults = $('#listing-actions-results-container');

				if($listingActionsResults.size()) {
					var fetchListingResultsTries = 0;

					var fetchListingResults = function() {
						$.post(
							BaT_Theme.ajaxUrl,
							{
								action:  BaT_Theme.ajaxActionListingResultsBox,
								listing: BaT_Theme.post
							},
							function(data) {
								if(data.is_ended) {
									$('#listing-actions-results-container').replaceWith(data.markup);
								} else {
									fetchListingResultsCaller();
								}
							}
						);
					};

					var fetchListingResultsCaller = function() {
						if(fetchListingResultsTries < 5) {
							fetchListingResultsTries++;
							setTimeout(fetchListingResults, 2000);
						}
					};

					fetchListingResultsCaller();
				}
			}

			if('undefined' !== typeof data.comment.end_timestamp) {
				updateCountdowns(data.comment.post, data.comment.end_timestamp);
			}

			CLVM.addComment(data.comment, true);
		}
	});

	// Comment deleted or moderated
	BAT_WEBSERVICES_PUSHER.bind(true, 'comment-deleted', function(data) {
		var CLVM = 'undefined' !== typeof BAT_VMS && 'undefined' !== typeof BAT_VMS.CLVM ? BAT_VMS.CLVM : false;

		if(CLVM) {
			CLVM.removeCommentById(data.comment_id);
		}
	});

	// Comment count updated for content item
	BAT_WEBSERVICES_PUSHER.bind(true, 'comments-updated', function(data) {
		$('[data-realtime_id="' + data.content + '"][data-realtime_prop="comments-updated.numeric"]').text(data.numeric);
		$('[data-realtime_id="' + data.content + '"][data-realtime_prop="comments-updated.formatted"]').text(data.formatted);
		$('[data-realtime_id="' + data.content + '"][data-realtime_prop="comments-updated.display"]').text(data.display).parents('.post-meta-comments').toggleClass('post-meta-comments-hide', 0 === data.numeric);
	});

	// Like count for a particular comment has been updated
	BAT_WEBSERVICES_PUSHER.bind(true, 'likes-updated-c', function(data) {
		var CLVM = 'undefined' !== typeof BAT_VMS && 'undefined' !== typeof BAT_VMS.CLVM ? BAT_VMS.CLVM : false;

		if(CLVM) {
			var comment = CLVM.getCommentById(data.comment_id);

			if(comment) {
				// Update the number of dislikes to hide comments as appropriate
				comment.dislikes(data.dislikes);

				// Update the number of likes for display
				comment.likes(data.numeric);
				comment.likesFormatted(data.formatted);

				// Update the users who have liked this comment to prevent double liking
				comment.likers(data.actors);
			}
		}
	});

	// Like count for a particular user has been updated
	BAT_WEBSERVICES_PUSHER.bind(true, 'likes-updated-u', function(data) {
		var CLVM = 'undefined' !== typeof BAT_VMS && 'undefined' !== typeof BAT_VMS.CLVM ? BAT_VMS.CLVM : false;

		if(CLVM) {
			var comments = CLVM.getCommentsByAuthor(data.user_id),
				index    = 0;

			for(index = 0; index < comments.length; index++) {
				var comment = comments[index];

				comment.authorLikes(data.numeric);
				comment.authorLikesFormatted(data.formatted);
			}
		}
	});

	// Get permission to push notifications for new listings
	if ('Notification' in window && BAT_WEBSERVICES_PUSHER.request_notifications) {
		if ('granted' !== Notification.permission && 'denied' !== Notification.permission) {
			Notification.requestPermission(function(permission) {
				if(!('permission' in Notification)) {
					Notification.permission = permission;
				}
			});
		}
	}

	// A new listing has been added
	BAT_WEBSERVICES_PUSHER.bind(true, 'listing-begin', function(data) {
		var listingLink = data.listing_link,
		    listingImg  = data.listing_img,
		    listingType = BAT_WEBSERVICES_PUSHER.listing_begin_title,
		    listingText = data.listing_text;

		if ('Notification' in window && BAT_WEBSERVICES_PUSHER.request_notifications) {
			if('granted' === Notification.permission) {
				var notification = new Notification(listingType, { body: listingText, icon: listingImg });

			} else if('denied' !== Notification.permission) {
				Notification.requestPermission(function(permission) {
					if(!('permission' in Notification)) {
						Notification.permission = permission;
					}

					if ('granted' === permission) {
						var notification = new Notification(listingType, { body: listingText, icon: listingImg });
					}
				});
			}
		}
	});

	// A new bid has been completed by a user
	BAT_WEBSERVICES_PUSHER.bind(true, 'completed-bid', function(data) {
		var theme = 'undefined' !== typeof BaT_Theme ? BaT_Theme : false;

		if(theme) {
			$('.recent-bids-list-item').removeClass('recent-bids-list-item-active');

			theme.addBid(data);
		}
	});

	// Metadata for a listing has been updated
	BAT_WEBSERVICES_PUSHER.bind(true, 'metadata-updated', function(data) {
		var theme = 'undefined' !== typeof BaT_Theme ? BaT_Theme : false;

		if(theme) {
			//FIXME Clean this up at some point
			if(data.withdrawn) {
				$('[data-listing-bid-id="' + data.post_id + '"] #listing-bid').parent().remove();
				$('[data-listing-intro-id="' + data.post_id + '"]').find('.listing-available-countdown').countdown('destroy');
				$('[data-listing-intro-id="' + data.post_id + '"]')
					.find('.listing-available-actions').empty().append($(data.listing_available_actions)).end()
					.find('.listing-available-info').empty().append($(data.listing_available_info)).end();

				$('.listing-actions-reserve').remove();
			}

			$('[data-listing-button="' + data.post_id + '"]').text(data.button_text);
			$('[data-listing-count="' + data.post_id + '"]').text(data.count_text);
			$('[data-listing-current="' + data.post_id + '"]').text(data.current_text);
			$('[data-listing-current-bidding="' + data.post_id + '"]').data('amount', data.current_numeric);
			$('[data-listing-current-bidding="' + data.post_id + '"]').data('high-bidder', data.high_bidder_id);
			$('[data-listing-current-bidding="' + data.post_id + '"]').data('minimum', data.minimum_bid_numeric);
			$('[data-listing-current-expanded="' + data.post_id + '"]').text(data.current_text_expanded);
			$('[data-listing-current-plain="' + data.post_id + '"]').text(data.current_text_plain);
			$('[data-listing-current-home="' + data.post_id + '"]').text(data.current_text_home);
			$('[data-listing-current-home-label="' + data.post_id + '"]').text(data.current_text_home_label);
			$('[data-listing-currently="' + data.post_id + '"]').replaceWith(data.currently);
			$('[data-listing-high-bidder-link="' + data.post_id + '"]').html(data.high_bidder_link);
			$('[data-listing-minimum="' + data.post_id + '"]').attr('placeholder', data.minimum_text);

			var $table = $('#listing-bid[data-listing-bid-id="' + data.post_id + '"]');

			if($table.length > 0) {
				$current_bid_row = $table.find('#current-bid-row');

				if('' === data.current_bid_row && $current_bid_row.length > 0) {
					$current_bid_row.remove();
				} else if('' !== data.current_bid_row && 0 === $current_bid_row.length) {
					$table.prepend(data.current_bid_row);
				}
			}
		}
	});


	BAT_WEBSERVICES_PUSHER.bind(true, 'reserve-updated', function(data) {
		var theme = 'undefined' !== typeof BaT_Theme ? BaT_Theme : false;

		if(theme) {
			var $listingActionsReserve = $('.listing-actions-reserve');

			if($listingActionsReserve.length > 0) {
				$.post(
					theme.ajaxUrl,
					{
						action:  theme.ajaxActionListingReserveBox,
						listing: data.item_id
					},
					function(data, status) {
						if(data.reserve_met) {
							$('#listing-action-reserve-warn').remove();
						}

						$('.listing-actions-reserve').replaceWith(data.markup);
					}
				);
			}
		}
	});

	BAT_WEBSERVICES_PUSHER.bind(true, 'metadata-updated', function(data) {
		$('[data-update="' + data.post_id + '"]').data('price', data.current_numeric);
	});

	BAT_WEBSERVICES_PUSHER.bind(true, 'moderate-comments-message-updated', function(data) {
		$('.comments-warning-container-comments-moderation[data-item="' + data.post_id + '"]').html(data.message);
	});

	// Ending timestamp for a listing has been updated
	BAT_WEBSERVICES_PUSHER.bind(true, 'endtime-updated', function(data) {
		$('[data-update="' + data.post_id + '"]').data('timestamp', data.end_timestamp);

		updateCountdowns(data.post_id, data.end_timestamp);
	});

	BAT_WEBSERVICES_PUSHER.bind(true, 'stats-updated', function(data) {
		if('undefined' !== typeof data.views) {
			$('[data-stats-id="' + data.post_id + '"][data-stats-item="views"]').text(data.views);
		}

		if('undefined' !== typeof data.watchers) {
			$('[data-stats-id="' + data.post_id + '"][data-stats-item="watchers"]').text(data.watchers);
		}

		if('undefined' !== typeof data.watchers_noun) {
			$('[data-stats-id="' + data.post_id + '"][data-stats-item="watchers_noun"]').text(data.watchers_noun);
		}

		if('undefined' !== typeof data.watchers_number) {
			$('[data-stats-id="' + data.post_id + '"][data-stats-item="watchers_number"]').text(data.watchers_number);
		}
	});

	BAT_WEBSERVICES_PUSHER.bind(true, 'votes-updated', function(data) {
		if('undefined' !== typeof data.item_id && 'undefined' !== typeof data.votes) {
			$('[data-votes-id="' + data.item_id + '"][data-votes-item="votes"]').text(data.votes);
		}

		if('undefined' !== typeof data.item_id && 'undefined' !== typeof window.galleryItems) {
			for(var i = 0; i < window.galleryItems.length; i++) {
				var item = window.galleryItems[i];

				if(item.id == data.item_id) {
					item.votes = data.votes;
				}
			}
		}
	});

	// Notify System of seller presence
	BAT_WEBSERVICES_PUSHER.bind('presence-seller-check', 'pusher:subscription_succeeded', function(members){
		members.each(function(member) {
			if(parseInt(member.info.post_id) == parseInt(BaT_Theme.post) && parseInt(BAT_VMS.postAuthor) == parseInt(member.id)) {
				$.post(
					BaT_Theme.ajaxUrl,
					{
						action:     BAT_WEBSERVICES_PUSHER.ajaxActionListingSellerPresence,
						listing_id: BaT_Theme.post,
						user_id:    BAT_VMS.postAuthor
					},
					function(data, status) {
						$('.postid-'+BaT_Theme.post+' #seller_last_present').text(BAT_WEBSERVICES_PUSHER.seller_online_title);
					}
				);
			}
		});
	});

	BAT_WEBSERVICES_PUSHER.bind('presence-seller-check', 'pusher:member_added', function(member){
		if(parseInt(member.info.post_id) == parseInt(BaT_Theme.post) && parseInt(BAT_VMS.postAuthor) == parseInt(member.id)) {
			$.post(
				BaT_Theme.ajaxUrl,
				{
					action:     BAT_WEBSERVICES_PUSHER.ajaxActionListingSellerPresence,
					listing_id: BaT_Theme.post,
					user_id:    BAT_VMS.postAuthor
				},
				function(data, status) {
					$('.postid-'+BaT_Theme.post+' #seller_last_present').text(BAT_WEBSERVICES_PUSHER.seller_online_title);
				}
			);
		}
	});

	BAT_WEBSERVICES_PUSHER.bind('presence-seller-check', 'pusher:member_removed', function(member){
		if(parseInt(member.info.post_id) == parseInt(BaT_Theme.post) && parseInt(BAT_VMS.postAuthor) == parseInt(member.id)) {
			$.post(
				BaT_Theme.ajaxUrl,
				{
					action:     BAT_WEBSERVICES_PUSHER.ajaxActionListingSellerPresence,
					listing_id: BaT_Theme.post,
					user_id:    BAT_VMS.postAuthor
				},
				function(data, status) {
					$('.postid-'+BaT_Theme.post+' #seller_last_present').text(data.date_current);
				}
			);
		}
	});
});
